//********************************************************************
//*-------------------------------------------------------------------
//* Sample Source
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2000, 2002
//*
//* The source code for this program is not published or otherwise
//* divested of its trade secrets, irrespective of what has been
//* deposited with the US Copyright Office.
//*-------------------------------------------------------------------
//*
////////////////////////////////////////////////////////////////////////////////

import com.ibm.commerce.catalog.commands.*;

import java.util.*;
import java.sql.*;
import java.rmi.*;
import javax.ejb.*;
import javax.naming.*;
import javax.servlet.http.HttpUtils.*;
import javax.servlet.http.*;

import com.ibm.commerce.datatype.*;
import com.ibm.commerce.base.objects.*;
import com.ibm.commerce.catalog.objects.*;

import com.ibm.commerce.ras.*;
import com.ibm.commerce.exception.*;
import com.ibm.commerce.command.*;
import com.ibm.commerce.server.*;

/**
 * This command first determines the type of CatalogEntry that is 
 * provided as input to the command.  If it is a Package, then the <I>PackageResolveSku</I> command is called.  If 
 * it is a Product, then first the command determines the distinct Attributes that are needed to resolve the Product 
 * into an orderable Item.  For each of these Attributes, the command ensures that an appropriate AttributeValue has 
 * been supplied as input.  Once these Attribute name-value pairs have been identified, the command resolves this 
 * information into an <I>ItemId</I> that represents the orderable Item.  Also, the command sets the <I>ItemSpecId</I> 
 * to be the ItemSpecification for the resolved CatalogEntry.  If Attributes are missing or are incorrect, 
 * the <I>ErrorMissingAttributeErrorCmd</I> or <I>ErrorProductAttributeErrorCmd</I> error tasks are launched 
 * respectively. If the type of CatalogEntry that is provided as input to the command is neither a Package nor a 
 * Product, then the command returns and sets the <I>ItemId</I> to be equal to the <I>ProductId</I>.
 */
public class ResolveSkuCmdImpl  extends TaskCommandImpl implements ResolveSkuCmd
{
	/**
	 * The internal copyright field.
	 */
	public static final String COPYRIGHT = com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;

	private Hashtable ihshAttributeNVPs = null;
	private Long inItemNumber = null;
	private Long inProductNumber = null;
	private Long inItemSpecId = null;

/**
 * Returns the hashtable that represents the Attribute names and values that will be used to perform the Item resolution. 
 * @return java.util.Hashtable
 */
protected Hashtable getAttributeNVPs() 
{
	return ihshAttributeNVPs;
}
private Enumeration getDistinctAttributes(ProductAccessBean aabProduct, Integer langId) throws com.ibm.commerce.exception.ECException 
{
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_CATALOG, this.getClass().getName(), "getDistinctAttributes");
	
	AttributeAccessBean arAttributeList[];
	Vector vecTemp = new Vector();
	
	try
	{
		arAttributeList = aabProduct.getAttributes(langId);

		if (arAttributeList.length == 0) // Product has no attributes
		{
			//The following check is to support tooling restirctions.
			//Here we allow for Products that do not have any attributes
			//but do have a PRODUCT_ITEM relationship defined.
			ItemAccessBean[] items = aabProduct.getItems();
			if (items.length != 0)
			{
				//We assume that there is only one Item defined so we use the first one
				setItemId(items[0].getCatalogEntryReferenceNumberInEJBType());
			}
			else
			{
			 
				// There aren't any distinct attributes, this is an error situation 
				//note: this is different behaviour than V4.1
			    ECTrace.trace(ECTraceIdentifiers.COMPONENT_CATALOG, this.getClass().getName(), "getDistinctAttributes", 
					"No distinct attributes are defined for this Product: " + getProductId().toString() 
					+ ". Calling error handler: " + ErrorMissingAttributeCmd.ERRTASK_NAME);	

				ECApplicationException exc =  new ECApplicationException(ECMessage._ERR_GETTING_SKU,
												  this.getClass().getName(),"getDistinctAttributes",
												  ErrorMissingAttributeCmd.ERRTASK_NAME);			
				throw exc;
			}	
		}

		
		for (int i=0; i < arAttributeList.length; i++)
		{
			String resolvable = arAttributeList[i].getUsage(); 
			if ( resolvable == null || resolvable.equals("1"))
			{
				//we know that this attribute is used for sku resolution, so add it to the list 
				vecTemp.addElement(arAttributeList[i]);
			}
		}		
	
	} catch (Exception e) {
		    throw new ECSystemException(ECMessage._ERR_GENERIC,this.getClass().getName(),"getDistinctAttributes",ECMessageHelper.generateMsgParms(e.getMessage()),e);
	}

	ECTrace.exit(ECTraceIdentifiers.COMPONENT_CATALOG, this.getClass().getName(), "getDistinctAttributes");

	return vecTemp.elements();
}
/**
 * Returns the ItemId that represents the primary key of the resolved Item
 * @return java.lang.Long
 */
public Long getItemId()
{
	return inItemNumber;
}
/**
 * Returns the ItemSpecId that represents the ItemSpecification identifier for the resolved Item.
 * @return java.lang.Long
 */
public Long getItemSpecId()
{
	return inItemSpecId;
}
/**
 * Returns the ProductId that represents the primary key of the CatalogEntry that is to be resolved,
 * @return java.lang.Long
 */
protected Long getProductId ()
{
	return inProductNumber;
}
public boolean isReadyToCallExecute()
{
	return true;
}
public void performExecute() throws ECException
{
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_CATALOG, this.getClass().getName(), "performExecute");
	super.performExecute();
		
	try
	{
		//Frist we need to make sure that this is a Product
		CatalogEntryAccessBean abCatEntry = new CatalogEntryAccessBean();
		abCatEntry.setInitKey_catalogEntryReferenceNumber(getProductId().toString());

		String strCatEntType = abCatEntry.getType();
		
		if (strCatEntType.equals(ECConstants.EC_PRODUCT_BEAN))
		{
			
			ProductAccessBean abProduct = new ProductAccessBean();
			abProduct.setInitKey_catalogEntryReferenceNumber(getProductId().toString());

			// Let's first get the list of Distinct Attributes for the product
			Enumeration enAttributeList = getDistinctAttributes(abProduct, commandContext.getLanguageId());

			//enAtributeList now contains a collection of distict attributes for the Product
			//we need to:
			//1. Determine if all distinct attributes have been specified by the caller and
			//2. If so, for each attribute, store its associated value (that was passed in)
			//   into a hashtable
			AttributeAccessBean abAttribute;
			String strValue;
			Hashtable hshResolveableAttributes = new Hashtable();
			int counter = 0;
			
			while (enAttributeList.hasMoreElements())
			{
				//Need to itterate through the list of distinct attributes
				//for each one, fish out the value from the passed in AttributeNVPs
				//if the attribute was not suplied, thow an error, else store the NVP in resolvableAttributes

				abAttribute = (AttributeAccessBean) enAttributeList.nextElement();
				Object val = getAttributeNVPs().get(abAttribute.getAttributeReferenceNumberInEJBType());
				if (val == null)
				{
					//the distinct attribute is missing
				    ECTrace.trace(ECTraceIdentifiers.COMPONENT_CATALOG, this.getClass().getName(), "performExecute", 
						"at least one distinct attribute is missing for product " + getProductId().toString() 
						+ ". Calling error handler: " + ErrorMissingAttributeCmd.ERRTASK_NAME 
						+ " with error code: " + ErrorMissingAttributeCmd._ERR_CODE_MISSING_ATTR);

					TypedProperty temp = new TypedProperty();
					temp.put(com.ibm.commerce.server.ECConstants.EC_ERROR_CODE, ErrorMissingAttributeCmd._ERR_CODE_MISSING_ATTR);
					temp.put(com.ibm.commerce.server.ECConstants.EC_PRODUCT, getProductId().toString());

					ECApplicationException exc =  new ECApplicationException(
														ECMessage._ERR_GETTING_SKU,
						                                this.getClass().getName(),"performExecute",
						                                ErrorMissingAttributeCmd.ERRTASK_NAME,
						                                temp
						                                );
				
					throw exc;								
				}
			    strValue = val.toString();

			    ECTrace.trace(ECTraceIdentifiers.COMPONENT_CATALOG, this.getClass().getName(), "performExecute", 
						"attribute is: " + abAttribute.getAttributeReferenceNumber() + " (" + abAttribute.getName() + ")");

				ECTrace.trace(ECTraceIdentifiers.COMPONENT_CATALOG, this.getClass().getName(), "performExecute", 
						"trying to match value: " + strValue);

				if (strValue == null)
				{
					//the distinct attribute is missing
				    ECTrace.trace(ECTraceIdentifiers.COMPONENT_CATALOG, this.getClass().getName(), "performExecute", 
						"The value for attribute: " + abAttribute.getAttributeReferenceNumber() + " is null." 
						+ " Calling error handler: " + ErrorMissingAttributeCmd.ERRTASK_NAME 
						+ " with error code: " + ErrorMissingAttributeCmd._ERR_CODE_MISSING_ATTR);
					
					TypedProperty temp = new TypedProperty();
					temp.put(com.ibm.commerce.server.ECConstants.EC_ERROR_CODE, ErrorMissingAttributeCmd._ERR_CODE_MISSING_ATTR);
					temp.put(com.ibm.commerce.server.ECConstants.EC_PRODUCT, getProductId().toString());

					ECApplicationException exc =  new ECApplicationException(
														ECMessage._ERR_GETTING_SKU,
						                                this.getClass().getName(),"performExecute",
						                                ErrorMissingAttributeCmd.ERRTASK_NAME,
						                                temp
						                                );
				
					throw exc;								
				}
				else
				{
					//add the attribute/value pair to the hashtable
					counter++;
					hshResolveableAttributes.put(abAttribute, strValue);
				}
				 
			}

			try
			{
				//Now that we have all the relevant atribute/value pairs, we can resolve the item number
				//But first we must check it the item has already been resolved
				if (getItemId() == null)
				{
					setItemId(abProduct.findSkuByAttributes(hshResolveableAttributes).getCatalogEntryReferenceNumberInEJBType());
				}
			
			} catch (Exception e) {
			
			    ECTrace.trace(ECTraceIdentifiers.COMPONENT_CATALOG, this.getClass().getName(), "performExecute", 
					"Sku resolution failed for product: " + getProductId()  
					+ " Calling error handler: " + ErrorProductAttributeCmd.ERRTASK_NAME 
					+ " with error code: " + ErrorProductAttributeCmd._ERR_CODE_BAD_PROD_ATTR);
			    
				TypedProperty temp = new TypedProperty();
				temp.put(com.ibm.commerce.server.ECConstants.EC_ERROR_CODE, ErrorProductAttributeCmd._ERR_CODE_BAD_PROD_ATTR);
				temp.put(com.ibm.commerce.server.ECConstants.EC_PRODUCT, getProductId().toString());

				ECApplicationException exc =  new ECApplicationException(
													ECMessage._ERR_GETTING_SKU,
					                                this.getClass().getName(),"performExecute",
					                                ErrorProductAttributeCmd.ERRTASK_NAME,
					                                temp
				 	                               );
				throw exc;								
			}
		}
		else if (strCatEntType.equals(ECConstants.EC_PACKAGE_BEAN))
		{
			//It's a package...call PackageResolveSku
		    ECTrace.trace(ECTraceIdentifiers.COMPONENT_CATALOG, this.getClass().getName(), "performExecute", 
				"Calling PackageResolveSkuCmd for Package: " + getProductId() );
		    
			PackageResolveSkuCmd comm = (PackageResolveSkuCmd) CommandFactory.createCommand(PackageResolveSkuCmd.NAME, getStoreId());
			comm.setPackageId(getProductId());
			comm.setAttributeNVPs(getAttributeNVPs());
			comm.setCommandContext(getCommandContext());
			comm.execute();
			setItemId(comm.getPackageItemId());
			
		}
		else if (strCatEntType.equals(ECConstants.EC_BUNDLE_BEAN))
		{
			//Bundles can not be directly processed, they need to be broken up into their invividual components
			throw new ECApplicationException(ECMessage._ERR_PROCESSING_BUNDLE, getClass().getName(),"performExecute");
		}
		else
		{
			//It is not a product or a package, so just set the
			//ItemId to the ProductId
		    ECTrace.trace(ECTraceIdentifiers.COMPONENT_CATALOG, this.getClass().getName(), "performExecute", 
				"The CatalogEntry is not a Product, Package or Bundle: " + getProductId() );
			setItemId(getProductId());
		}

	    ECTrace.trace(ECTraceIdentifiers.COMPONENT_CATALOG, this.getClass().getName(), "performExecute", 
						"Resolution complete, attempting to set the item spec for catentry: " + getItemId().toString());

		//Set the ItemSpecification (if there is one)
	    try 
	    {
		    
			abCatEntry = new CatalogEntryAccessBean();
			abCatEntry.setInitKey_catalogEntryReferenceNumber(getItemId().toString());
		    setItemSpecId(abCatEntry.getItemspc_idInEJBType());
			    
	    } catch (Exception e) {
		  //Ignore exception, allows for non-ATP function  
		  //  ECMessageLog.out(ECMessage._DBG_NO_BASE_ITEM_FOUND, this.getClass().getName(), 
		  //	    "performExecute", ECMessageHelper.generateMsgParms(getItemId()));
	    }

	    ECTrace.trace(ECTraceIdentifiers.COMPONENT_CATALOG, this.getClass().getName(), "performExecute", 
						"Item Specification set: " + getItemSpecId());
			
	} catch (ECException e) {
		throw e;
	} catch (ObjectNotFoundException e) {
			throw new ECSystemException(ECMessage._ERR_PROD_NOT_EXISTING,this.getClass().getName(),"performExecute",ECMessageHelper.generateMsgParms(getProductId()),e);
	} catch (Exception e) 	{
		
		ECTrace.trace(ECTraceIdentifiers.COMPONENT_CATALOG, this.getClass().getName(), "performExecute","ResolveSKU trace point#27");
		ECTrace.trace(ECTraceIdentifiers.COMPONENT_CATALOG, this.getClass().getName(), "performExecute","in catch block, product is: "
			                                                                           + getProductId() + " getAttributeNVPs: " 
			                                                                           + getAttributeNVPs());
		
	    throw new ECSystemException(ECMessage._ERR_GENERIC,this.getClass().getName(),"performExecute",ECMessageHelper.generateMsgParms(e.getMessage()),e);
	}
		

	ECTrace.exit(ECTraceIdentifiers.COMPONENT_CATALOG, this.getClass().getName(), "performExecute");
}
public void reset() {
	super.reset();

	ihshAttributeNVPs = null;
	inItemNumber = null;
	inProductNumber = null;
	inItemSpecId = null;
}
/**
 * Sets the hashtable that represents the Attribute names and values that will be used to perform the Item resolution. 
 * @param ahshAttrNVPs java.util.Hashtable
 */
public void setAttributeNVPs(Hashtable ahshAttrNVPs) 
{
	ihshAttributeNVPs = ahshAttrNVPs;	
}
/**
 * Sets the ItemId that represents the primary key of the resolved Item
 * @param anItemNumber java.lang.Long
 */
protected void setItemId (Long anItemNumber)
{
	inItemNumber = anItemNumber;
}
/**
 * Sets the ItemSpecId that represents the ItemSpecification identifier for the resolved Item.
 * @param anItemSpecId java.lang.Long
 */
protected void setItemSpecId (Long anItemSpecId)
{
	inItemSpecId = anItemSpecId;
}
/**
 * Sets the ProductId that represents the primary key of the CatalogEntry that is to be resolved,
 * @param anProductId java.lang.Long
 */
public void setProductId (Long anProductNumber)
{
	inProductNumber = anProductNumber;
}
public void validateParameters() throws ECException 
{
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_CATALOG, this.getClass().getName(), "validateParameters");
	
	super.validateParameters();

	if (getProductId() == null) 
		throw new ECSystemException(ECMessage._ERR_DIDNT_SET_PRODUCT, this.getClass().getName(), "validateParameters");

	if (getAttributeNVPs() == null)
		setAttributeNVPs(new Hashtable());
		
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_CATALOG, this.getClass().getName(), "validateParameters");
}

/**
 * No value constructor.
 */
public ResolveSkuCmdImpl() 
{
	super();
}
}
