//********************************************************************
//*-------------------------------------------------------------------
//* Sample Source
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2000, 2002
//*
//* The source code for this program is not published or otherwise
//* divested of its trade secrets, irrespective of what has been
//* deposited with the US Copyright Office.
//*-------------------------------------------------------------------
//*
////////////////////////////////////////////////////////////////////////////////

import com.ibm.commerce.fulfillment.commands.*;

import com.ibm.commerce.inventory.objects.*;
import java.sql.*;

import com.ibm.commerce.datatype.*;
import java.util.*;
import java.rmi.*;
import javax.ejb.*;
import javax.naming.*;

import com.ibm.commerce.base.objects.*;
import com.ibm.commerce.catalog.objects.*;
import com.ibm.commerce.fulfillment.objects.*;
import com.ibm.commerce.order.objects.*;
import com.ibm.commerce.order.utils.*;
import com.ibm.commerce.fulfillment.objects.*;
import com.ibm.commerce.common.objects.*;

import com.ibm.commerce.ras.*;
import com.ibm.commerce.exception.*;
import com.ibm.commerce.command.*;
import com.ibm.commerce.user.*;
import com.ibm.commerce.server.*;
import com.ibm.commerce.utils.*;

import com.ibm.commerce.returns.commands.*;
import com.ibm.commerce.price.utils.*;

/**
 * Allocates existing inventory for a particular order item at a particular fulfillment center.
 *
 * <p><strong>Behaviour</strong>
 * <p>The behaviour of the command is:
 * <ol><li>Set next strategy to input strategy.
 * <li>If the order item is suitable for allocation (i.e. its inventory status attribute specifies 
 * unallocated or backordered), and there is sufficient existing inventory at one of the specified 
 * fulfillment centers (searching in order from first to last in the input fulfillment centers), 
 * to completely allocate the quantity needed by the order item, allocate it and update the order item 
 * to reflect the allocation.
 * <li>If the order item is still unallocated or backordered and there is sufficient existing inventory 
 * at one of the specified fulfillment centers (searching in order from first to last in the 
 * list of input fulfillment centers), to partially or fully allocate the quantity needed by the order 
 * item, allocate that amount, rounded down as explained in the next sentence, and update the order item 
 * to reflect the allocation.  When the amount to allocate cannot be allocated fully, the amount actually 
 * allocated is rounded down to the nearest multiple of a floating point number obtained by converting 
 * the CatEntShip.QuantityMultiple for the CatEntry of the order item to the unit of measure specified 
 * by BaseItem, and divided by BaseItem.QuantityMultiple, and then that multiple is rounded to the 
 * nearest integer.  When there is no CatEntShip for the order item's CatEntry, CatEntShip.QuantityMultiple 
 * is assumed to be 1, and CatEntShip.QuantityMeasure is assumed to be 'C62'.  However, when the order item 
 * has no CatEntry, the amount to allocate is not rounded down at all (i.e. it is rounded down to the 
 * nearest multiple of 1).
 * <li>If none of the needed quantity can be allocated, the order item is not changed.
 *
 */
public class AllocateExistingInventoryCmdImpl extends TaskCommandImpl implements AllocateExistingInventoryCmd
{
	public static final String COPYRIGHT                 = com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;
	protected static String istrThisClass                = "com.ibm.commerce.order.commands.AllocateExistingInventoryCmdImpl";	
	protected static String istrPerformFunc              = "performExecute";
	protected static String istrCheckParametersFunc      = "checkParameters";

	// input parameters
	private Vector              ivFulfillmentCenters     = null;
	private OrderItemAccessBean iabOrderItem             = null;
	private Integer             inStrategy 				 = InventoryManagementHelper.DEFAULT_STRATEGY;
	
	private Integer             inNextStrategy 		   = InventoryManagementHelper.DEFAULT_STRATEGY;

	private boolean allocateFromAllFFCs(OrderItemAccessBean anabOrderItem, String strPartialAllocationFlag, Double afAllocationMultiple) throws RemoteException, NamingException, FinderException, CreateException, SQLException, InvalidParameterValueException, ParameterNotFoundException, ECApplicationException
	{
		boolean      rc          = false;
		final String methodName = "allocateFromAllFFCs";

		ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, istrThisClass, methodName);
		
		Enumeration  enFFCs     = getFulfillmentCenters().elements();
			
		while (enFFCs.hasMoreElements())
		{
			Integer nFulfillmentCenter = (Integer) enFFCs.nextElement();
			Long    nOrderItemsId      = anabOrderItem.getOrderItemIdInEJBType();
						
			TypedProperty tpSPResult = StoredProcedureHelperProxy.callGetItems(nOrderItemsId, nFulfillmentCenter, strPartialAllocationFlag, afAllocationMultiple);

			Integer nQtyAllocated  = tpSPResult.getInteger(InventoryManagementHelper.QUANTITY);
			String  strStatus      = tpSPResult.getString (InventoryManagementHelper.STATUS  );
			
			if (!strStatus.equals(InventoryManagementHelper.STATUS_SUCCESS))
			{
			   // item is discontinued, marked for deletion or not backorderable				
				ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, istrThisClass, methodName, "Could not allocate inventory for order item " + nOrderItemsId + " from fulfillment center " + nFulfillmentCenter + "; status : " + strStatus);
			}			

			if (nQtyAllocated.intValue()==0)
			  continue;
			  
			ECMessageLog.out(ECMessage._STA_ALLOC_QUANTITY, istrThisClass, methodName, ECMessageHelper.generateMsgParms(nQtyAllocated, anabOrderItem.getOrderItemId(), nFulfillmentCenter));
			  
			InventoryManagementHelper.setOrderItemAllocationInfo(anabOrderItem, nFulfillmentCenter, nQtyAllocated, InventoryManagementHelper.INV_ALLOCATED, TimestampHelper.now().toString());
			rc = true;
			break;									
			
		}

		ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, istrThisClass, methodName);

		return rc;
	}
/**
 * Gets a prioritized list of fulfillment centers from which the inventory should be allocated.
 */
public java.util.Vector getFulfillmentCenters() {
	return ivFulfillmentCenters;
}
/**
 * Gets the next allocation strategy set by the command.
 */
public java.lang.Integer getNextStrategy() {
	return inNextStrategy;
}
/**
 * Gets the order item for which existing inventory is to be allocated.
 *
 */
public com.ibm.commerce.order.objects.OrderItemAccessBean getOrderItem() {
	return iabOrderItem;
}
/**
 * Gets the next allocation strategy set by the command.
 */
public java.lang.Integer getStrategy() {
	return inStrategy;
}
/**
 * Contains the actual business logic of the command.
 */
public void performExecute() throws ECException
{
	final String methodName = istrPerformFunc;
	
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, istrThisClass, methodName);

//	checkParameters();	

	try
	{	
		TypedProperty       tpSPResult        = null;
		OrderItemAccessBean abOrderItem       = getOrderItem();
		Long                nOrderItemsId     = abOrderItem.getOrderItemIdInEJBType   ();
		Integer             nQuantityRequired = abOrderItem.getNeededQuantityInEJBType();

		
		boolean             bPartialAllocationAllowed = abOrderItem.getStatus().equals(OrderConstants.ORDER_BACKORDERED) ? false : true;

		inNextStrategy = inStrategy; 
		
		// do nothing if an item is already allocated
		String strInventoryStatus = abOrderItem.getInventoryStatus();
		if (strInventoryStatus == null)
		{
			strInventoryStatus = InventoryManagementHelper.INV_NOT_ALLOCATED;
			abOrderItem.setInventoryStatus(strInventoryStatus);
			abOrderItem.commitCopyHelper  ();								
		}
		
		if (!strInventoryStatus.equals(InventoryManagementHelper.INV_NOT_ALLOCATED) && !strInventoryStatus.equals(InventoryManagementHelper.INV_BACKORDERED))
		{			
			ECMessageLog.out(ECMessage._STA_CANNOTALLOC, istrThisClass, methodName, ECMessageHelper.generateMsgParms(nOrderItemsId, strInventoryStatus));
			return;
		}
		 		
		Enumeration   enFFCs = getFulfillmentCenters().elements();
				
		Double fAllocationMultiple = new Double(1.0); // if we must allocate all the needed inventory allocation multiple should be 1

		// attempt to allocate all of the needed inventory from a single fulfillment center
		if (!allocateFromAllFFCs(abOrderItem, InventoryManagementHelper.NO_PARTIAL_ALLOCATION, fAllocationMultiple))
		{
			// failed to allocate all from one ffc; attempt to allocate partial amount from one of the ffcs
			// PARTIAL ALLOCATION IS NOT ALLOWED FOR ORDER ITEMS WITH STATUS 'B' (i.e., backordered orders)
			// calculated the quantity multiple used when allocating inventory; the amount allocated is rounded down
			// to the nearest number of this quantityMultiple
			// quantityMultiple is obtained by converting the CatEntShip.QuantityMultiple for the CatEntry in the
			// order item to the unit of measure specified by BaseItem, divided by BaseItem.QuantityMultiple, and
			// rounded to the nearest integer
			if (bPartialAllocationAllowed)
			{
			  fAllocationMultiple = InventoryManagementHelper.calculateInventoryAllocationMultiple(abOrderItem);
			  allocateFromAllFFCs(abOrderItem, InventoryManagementHelper.PARTIAL_ALLOCATION, fAllocationMultiple);
			}
		}					

	}
	catch (RemoteException e) { throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, istrThisClass, methodName, ECMessageHelper.generateMsgParms( e.getMessage()), e );}
	catch (CreateException e) { throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, istrThisClass, methodName, ECMessageHelper.generateMsgParms( e.getMessage()), e );}
	catch (NamingException e) { throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, istrThisClass, methodName, ECMessageHelper.generateMsgParms( e.getMessage()), e );}
	catch (FinderException e) { throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, istrThisClass, methodName, ECMessageHelper.generateMsgParms( e.getMessage()), e );}
	catch (SQLException    e) { throw new ECSystemException(ECMessage._ERR_SQL_EXCEPTION   , istrThisClass, methodName, ECMessageHelper.generateMsgParms( e.getMessage()), e );}

	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, istrThisClass, methodName);
}
/**
 * This method should be called after a command has been executed to reset its states variables.
 */	
public void reset() 
{
		ivFulfillmentCenters     = null;
		iabOrderItem             = null;		
		inStrategy               = InventoryManagementHelper.DEFAULT_STRATEGY;
		
		inNextStrategy 		     = InventoryManagementHelper.DEFAULT_STRATEGY;
}
/**
 * Sets a prioritized list of fulfillment centers from which the inventory should be allocated.
 */
public void setFulfillmentCenters(java.util.Vector newFulfillmentCenters) {
	ivFulfillmentCenters = newFulfillmentCenters;
}
/**
 * Sets the next allocation strategy.
 */
public void setNextStrategy(java.lang.Integer newNextStrategy) {
	inNextStrategy = newNextStrategy;
}
/**
 * Sets the order item for which existing inventory is to be allocated.
 *
 */
public void setOrderItem(com.ibm.commerce.order.objects.OrderItemAccessBean newOrderItem) {
	iabOrderItem = newOrderItem;
}
/**
 * Sets allocation strategy to be used. Zero must be an acceptable value.
 */	
public void setStrategy(java.lang.Integer newStrategy) {
	inStrategy = newStrategy;
}
/**
 * Performs server side parameter checking.
 */
public void validateParameters() throws com.ibm.commerce.exception.ECException {
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, istrThisClass, istrCheckParametersFunc);

	if (getOrderItem() == null)
		throw new ECApplicationException(ECMessage._ERR_BAD_MISSING_CMD_PARAMETER, istrThisClass, istrCheckParametersFunc, ECMessageHelper.generateMsgParms(OrderConstants.EC_ORDER_ID));

	if (getFulfillmentCenters() == null)
		throw new ECApplicationException(ECMessage._ERR_BAD_MISSING_CMD_PARAMETER, istrThisClass, istrCheckParametersFunc, ECMessageHelper.generateMsgParms(OrderConstants.EC_FULFILLMENT_CENTER_ID));

	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, istrThisClass, istrCheckParametersFunc);
}
}
