//********************************************************************
//*-------------------------------------------------------------------
//* Sample Source
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2000, 2002
//*
//* The source code for this program is not published or otherwise
//* divested of its trade secrets, irrespective of what has been
//* deposited with the US Copyright Office.
//*-------------------------------------------------------------------
//*
////////////////////////////////////////////////////////////////////////////////

import com.ibm.commerce.payment.commands.*;

import java.sql.*;
import java.math.*;
import java.util.*;
import java.util.Date;

import com.ibm.commerce.order.objects.*;

import com.ibm.commerce.user.*;

import com.ibm.commerce.ras.*;
import com.ibm.commerce.server.*;
import com.ibm.commerce.command.*;
import com.ibm.commerce.exception.*;
import com.ibm.commerce.copyright.IBMCopyright;
import com.ibm.commerce.datatype.TypedProperty;
import com.ibm.commerce.order.commands.OrderProcessCmdImpl;
import com.ibm.commerce.order.utils.OrderConstants;
import com.ibm.commerce.payment.objects.*;	



/**
 * This <code>DoPaymentTestCmdImpl</code> Task Command Implementation is called by 
 * the <code>OrderProcessCmd</code> Controller Command to process payment 
 * for an order.
 * <p>
 * If the command is successful, the <code>OrderProcessCmd</code> will set the Order
 * status to the value specified by this command.
 * <p>
 * If unsuccessful, the command throws the ECApplicationException
 * specifying an Error Code and the Error View named by the constant
 * com.ibm.commerce.DoPaymentCmd.ERROR_TASK_NAME.
 * <p>
 * This implementation of the <code>DoPaymentCmd</code> Task Command does 
 * not have any returnData for the <code>OrderProcessCmd</code>.
 *
 */
public class DoPaymentTestCmdImpl  extends BusinessPolicyCommandImpl implements DoPaymentPolicyCmd
{
	private final static String COPYRIGHT 				= IBMCopyright.SHORT_COPYRIGHT;
	private final static String CLASSNAME 				= DoPaymentTestCmdImpl.class.getName();

	private final static int    MAX_PAYMETHOD_LEN   	= 5;
		
	private Long            	itsOrderId				= null;
	private OrderAccessBean 	iabOrder                = null;
	private AccountAccessBean 	iabAccount            	= null;	
	private BigDecimal 			itsTotalAmount  		= null;
	private String  			itsCardBrand       		= null;
	private String  			itsCardNumber     		= null;
	private Integer 			itsCardExpiryMonth    	= null;
	private Integer 			itsCardExpiryYear    	= null;
	private TypedProperty 		itsPaymentAttributes    = null;
	
	private Long            	inBuyerPOId             = null;
	private String          	isErrorViewName         = ERRTASK_NAME;

	private String          	istrOrderState          = null;
	
	/**
	 * Checks the card expiry date provided by the customer.
	 * 
	 * @exception ECException Throws an ECApplicationException, a subclass of ECException,
	 * if the expiry date is invalid.  
	 * The exceptionData includes the key-value pair of 
	 * ECConstants.EC_ERROR_CODE and DoPaymentCmd.ERR_CODE_BAD_XDATE.
	 */
	private void checkCardExpiry() throws ECException {
		final String methodName = "checkCardExpiry";
		ECTrace.entry( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName );

		int month = getCardExpiryMonth().intValue();
		int year  = getCardExpiryYear() .intValue();
		
		if ( ! isCardExpiryDateValid( month, year ) )
		{
			TypedProperty nvps = new TypedProperty();
			String msg = ECMessageHelper.getUserMessage(ECMessage._ERR_PAY_EXPIRY_DATE_INVALID,
											null, getCommandContext().getLocale() );
			nvps.put( DoPaymentCmd.MESSAGE_FOR_BUYER, msg );
			nvps.put( ECConstants.EC_ERROR_CODE, PaymentConstants.ERR_CODE_BAD_XDATE );
			throw new ECApplicationException(
							ECMessage._DBG_API_DO_PAYMENT_BAD_XDATE, CLASSNAME, methodName,
							ECMessageHelper.generateMsgParms( getCardExpiryYear().toString(),
								getCardExpiryMonth().toString() ),
							ERRTASK_NAME, nvps );
		}	

		ECTrace.exit( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName );
	}
	/**
	 * Checks the card number provided by the customer.
	 * This method calls the <code>CheckCCNumberCmd</code> Task Command to check
	 * the card number.
	 *
	 * @exception ECException Throws an ECApplicationException, a subclass of ECException,
	 * if the card number is invalid. 
	 */
	private void checkCardNumber() throws ECException {
		final String methodName = "checkCardNumber";
		ECTrace.entry( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName );

		CheckCCNumberCmd cmd = null;
		
		cmd = (CheckCCNumberCmd) CommandFactory.createCommand( CheckCCNumberCmd.NAME, getStoreId() );

		cmd.setCardBrand     ( getCardBrand ()     );
	  	cmd.setCardNumber    ( getCardNumber()     );
		cmd.setErrorTaskName ( ERRTASK_NAME        );
	   	cmd.setCommandContext( getCommandContext() );
	   	cmd.execute();

		ECTrace.exit( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName );
	}
/**
 * Checks the validity of customer-supplied parameters.
 */
protected void checkParameters() throws ECException
{
	final String methodName = "checkParameters";
	ECTrace.entry( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName );

	// Check for User Input Error and throws ECApplicationException....
	//
	TypedProperty paymentAttributes = getPaymentAttributes();
		
	setCardBrand ( paymentAttributes.getString(ECConstants.EC_CC_TYPE, ECConstants.EC_STR_EMPTY) );
	if ( getCardBrand().trim().length() == 0 )
	{
		TypedProperty nvps  = new TypedProperty();
		String msg = ECMessageHelper.getUserMessage(ECMessage._ERR_PAY_CARD_BRAND_MISSING,
										null, getCommandContext().getLocale() );
		nvps.put( DoPaymentCmd.MESSAGE_FOR_BUYER, msg );
		nvps.put( ECConstants.EC_ERROR_CODE	, PaymentConstants.ERR_CODE_MISSING_PARAM );
		nvps.put( DoPaymentCmd.NVP_FIELD	, ECConstants.EC_CC_TYPE );
		throw new ECApplicationException(
						ECMessage._ERR_PAY_CARD_BRAND_MISSING, CLASSNAME, methodName,
						ERRTASK_NAME, nvps ); 
	}

	setCardNumber( paymentAttributes.getString(ECConstants.EC_CC_NUMBER, ECConstants.EC_STR_EMPTY) );
	if ( getCardNumber().trim().length() == 0 )
	{
		TypedProperty nvps  = new TypedProperty();
		String msg = ECMessageHelper.getUserMessage(ECMessage._ERR_PAY_CARD_NUMBER_MISSING,
										null, getCommandContext().getLocale() );
		nvps.put( DoPaymentCmd.MESSAGE_FOR_BUYER, msg );
		nvps.put( ECConstants.EC_ERROR_CODE	, PaymentConstants.ERR_CODE_MISSING_PARAM   );
		nvps.put( DoPaymentCmd.NVP_FIELD	, ECConstants.EC_CC_NUMBER );
		throw new ECApplicationException(
						ECMessage._ERR_PAY_CARD_NUMBER_MISSING, CLASSNAME, methodName,
						ERRTASK_NAME, nvps );
	}

	try { setCardExpiryMonth( paymentAttributes.getInteger(ECConstants.EC_CCX_MONTH) ); }
	catch ( com.ibm.commerce.exception.ParameterNotFoundException ex ) { 
		TypedProperty nvps  = new TypedProperty();
		String msg = ECMessageHelper.getUserMessage(ECMessage._ERR_PAY_EXPIRY_MONTH_MISSING,
										null, getCommandContext().getLocale() );
		nvps.put( DoPaymentCmd.MESSAGE_FOR_BUYER, msg );
		nvps.put( ECConstants.EC_ERROR_CODE, PaymentConstants.ERR_CODE_MISSING_PARAM   );
		nvps.put( DoPaymentCmd.NVP_FIELD,    ECConstants.EC_CCX_MONTH );
		throw new ECApplicationException( ECMessage._ERR_PAY_EXPIRY_MONTH_MISSING, CLASSNAME, methodName,
						ERRTASK_NAME, nvps );
	}
	catch ( com.ibm.commerce.exception.InvalidParameterValueException ex ) { 
		TypedProperty nvps  = new TypedProperty();
		String msg = ECMessageHelper.getUserMessage(ECMessage._ERR_PAY_EXPIRY_DATE_INVALID,
										null, getCommandContext().getLocale() );
		nvps.put( DoPaymentCmd.MESSAGE_FOR_BUYER, msg );
		nvps.put( ECConstants.EC_ERROR_CODE, PaymentConstants.ERR_CODE_BAD_XDATE       );
		nvps.put( DoPaymentCmd.NVP_FIELD,    ECConstants.EC_CCX_MONTH );
		throw new ECApplicationException( ECMessage._ERR_PAY_EXPIRY_DATE_INVALID, CLASSNAME, methodName,
						ERRTASK_NAME, nvps );
	}

	try { setCardExpiryYear ( paymentAttributes.getInteger(ECConstants.EC_CCX_YEAR) ); }
	catch ( com.ibm.commerce.exception.ParameterNotFoundException ex ) { 
		TypedProperty nvps  = new TypedProperty();
		String msg = ECMessageHelper.getUserMessage(ECMessage._ERR_PAY_EXPIRY_YEAR_MISSING,
										null, getCommandContext().getLocale() );
		nvps.put( DoPaymentCmd.MESSAGE_FOR_BUYER, msg );
		nvps.put( ECConstants.EC_ERROR_CODE, PaymentConstants.ERR_CODE_MISSING_PARAM );
		nvps.put( DoPaymentCmd.NVP_FIELD,    ECConstants.EC_CCX_YEAR );
		throw new ECApplicationException( ECMessage._ERR_PAY_EXPIRY_YEAR_MISSING, CLASSNAME, methodName,
						ERRTASK_NAME, nvps );
	}
	catch ( com.ibm.commerce.exception.InvalidParameterValueException ex ) { 
		TypedProperty nvps  = new TypedProperty();
		String msg = ECMessageHelper.getUserMessage(ECMessage._ERR_PAY_EXPIRY_DATE_INVALID,
										null, getCommandContext().getLocale() );
		nvps.put( DoPaymentCmd.MESSAGE_FOR_BUYER, msg );
		nvps.put( ECConstants.EC_ERROR_CODE, PaymentConstants.ERR_CODE_BAD_XDATE );
		nvps.put( DoPaymentCmd.NVP_FIELD,    ECConstants.EC_CCX_YEAR );
		throw new ECApplicationException( ECMessage._ERR_PAY_EXPIRY_DATE_INVALID, CLASSNAME, methodName,
						ERRTASK_NAME, nvps );
	}

	ECTrace.exit( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName );
}
	public AccountAccessBean getAccount () {return iabAccount;}
	protected Long getAccountId ()
			throws javax.ejb.CreateException,javax.ejb.FinderException,
					javax.naming.NamingException,java.rmi.RemoteException {
		if (getAccount() == null) {
			return null;
		} else {
			return getAccount().getAccountIdInEJBType();
		}
	}
	protected Long getBuyerPOId () {
		return inBuyerPOId;
	}
protected String getCardBrand()
{
	return itsCardBrand;
}
protected Integer getCardExpiryMonth()
{
	return itsCardExpiryMonth;
}
protected Integer getCardExpiryYear()
{
	return itsCardExpiryYear;
}
protected String getCardNumber()
{
	return itsCardNumber;
}
/**
 * Returns the Content Type (or MIME type) of the return data, or <code>null</code> if
 * the type is not known.
 *
 * @return a <code>String</code> containing the name of the MIME type of the return data, 
 * or null if the type is not known.  Since this command implementation does not have
 * return data, the method returns <code>null</code>.
 */
public String getContentType() 
{
	return null;
}
	protected String getErrorViewName () { return isErrorViewName; }
	/**
	 * This method will first check if the local variable is set or not.
	 * If not set it will attempt to get it by create a new AccessBean.
	 */
	protected OrderAccessBean getOrder () {
		return iabOrder;
	}
/**
 * Returns the single character String value that the called command
 * wants the Calling command to use to set the Status of the order.
 *
 * If the returned String is empty or null, the Calling command will 
 * decide the value to use for the Status
 *
 * @return java.lang.String
 */
public String getOrderState() 
{
	return istrOrderState;
}
protected TypedProperty getPaymentAttributes()
{
	return itsPaymentAttributes;
}
/**
 * Returns the byte Array containing Return Data from the Payment System.
 * 
 * The Payment System used by this command does not use Return Data.  Therefore return null.
 * 
 * @return byte[]
 */
public byte[] getReturnData() 
{
	return null;
}
protected BigDecimal getTotalAmount()
{
	return itsTotalAmount;
}
/**
 * Checks if the specified expiry date (expiryMonth and expiryYear) is valid.
 * 
 * @return ture if the expiry date is valid.
 * @param expiryMonth the calender month of the expiry date (1 for January, 2 for February, and so on.)
 * @param expiryYear the calender year of the expiry date
 */
public static boolean isCardExpiryDateValid(int expiryMonth, int expiryYear) 
{
	final String methodName 	= "isCardExpiryDateValid";

	final int CC_VALID_Y_RANGE  = 10; 	// No card lasts this many years!
		
	Calendar currentDate 	= Calendar.getInstance();
	Calendar expDate 		= Calendar.getInstance();
	expDate.clear();

	if ( expiryMonth < 1 || expiryMonth > 12 )
	{
		ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName, 
			"Month is invalid.");
		return false;
	}
	
	// Sets the Expiry Date to the 1st day of the following month after expiryMonth.
	// Note that this is correct since the Calendar class takes month field values
	// from 0 to 11 with 0 meaning month of January. Thus, the following stmts are 
	// equivalent:   expDate.set( 1999, 12, 1 )                  
	//               expDate.set( 2000,  0, 1 )  <== January 1st, Year 2000.
	//    
	expDate.set( expiryYear, expiryMonth, 1 );		

	if ( expDate.before(currentDate) )
	{
		ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName, 
			"Date has already expired ");
		return false;
	}

	expDate.clear();
	expDate.set( expiryYear - CC_VALID_Y_RANGE, expiryMonth, 1 );

	if ( expDate.after(currentDate) )
	{
		ECTrace.trace( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName, 
			"Date too far ahead " );
		return false; 	
	}
	
	return true;		// Dropping through mean CardExpiryDate is valid.
}
/**
 * This method is called by the Command Framework to check if all basic mandatory
 * parameters have been set for this command.  This is done before the Framework calls
 * the performExecute method of the command.
 */
public boolean isReadyToCallExecute() 
{
	final String methodName = "isReadyToCallExecute";
	ECTrace.entry( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName );

	if ( getOrder() == null )
	{
		ECMessageLog.out( ECMessage._ERR_DIDNT_SET_ORDER, CLASSNAME, methodName );
		return false;
	}
	if ( getTotalAmount() == null )
	{
		ECMessageLog.out( ECMessage._ERR_DIDNT_SET_TOTAL_AMOUNT, CLASSNAME, methodName );
		return false;
	}

	if ( getPaymentAttributes()==null )
	{
		ECMessageLog.out( ECMessage._ERR_DIDNT_SET_PAYMENT_ATTRIB, CLASSNAME, methodName );
		return false;
	}

	ECTrace.exit( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName );
	return true;
}
	/**
	 * Performs the main business logic of the command.
	 */
	public void performExecute() throws ECException {
		final String methodName = "performExecute";
		ECTrace.entry( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName );

		checkCardNumber();			// Throws ECApplicationException if Card Number is Bad

		checkCardExpiry();			// Throws ECApplicationException if Card Expiry Date is Bad

		try {
			String encryptFlag = WcsApp.configProperties.getValue( "Instance/PDIEncrypt");
			boolean encrypt = encryptFlag.equalsIgnoreCase("ON") ? true : false;

			String strPayMethod = ( getCardBrand().length() > MAX_PAYMETHOD_LEN ) ? 
										getCardBrand().substring( 0, MAX_PAYMETHOD_LEN )
										: getCardBrand();
				
			Calendar expDate = Calendar.getInstance();
			expDate.clear();
				// Sets the Expiry Date to the 1st day of the following month after expiry month.
				// Note that this is correct since the Calendar class takes month field values
				// from 0 to 11 with 0 meaning month of January. Thus, the following stmts are 
				// equivalent:   expDate.set( 1999, 12, 1 )                  
				//               expDate.set( 2000,  0, 1 )  <== January 1st, Year 2000.
				//    
			expDate.set( getCardExpiryYear().intValue(), getCardExpiryMonth().intValue(), 1 );		
			expDate.add( java.util.Calendar.MILLISECOND, -1 );	// Go Back 1 Millisecond.
			Date date = expDate.getTime();
			ECTrace.trace( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName, 
				"Card Expiry Date: " + date.toString() );


			// Create new OrdPayMthd object
			com.ibm.commerce.order.objimpl.OrderPaymentMethodInputData bOrderPaymentMethodInputData = new com.ibm.commerce.order.objimpl.OrderPaymentMethodInputData();
			bOrderPaymentMethodInputData.setOrderId(getOrder().getOrderIdInEJBType());
			bOrderPaymentMethodInputData.setPaymentMethodId(strPayMethod);
			bOrderPaymentMethodInputData.setPaymentDevice(getCardNumber());

			bOrderPaymentMethodInputData.setEndDate( new Timestamp(date.getTime()) );
			bOrderPaymentMethodInputData.setMaximumAuthorizationAmount(getTotalAmount());
			bOrderPaymentMethodInputData.setAccountId(getAccountId());
			bOrderPaymentMethodInputData.setBuyerPurchaseOrderId(getBuyerPOId());
			bOrderPaymentMethodInputData.setPolicyId(getPolicyId());

			OrderPaymentMethodAccessBean abOrdPayMthd = new OrderPaymentMethodAccessBean(bOrderPaymentMethodInputData, encrypt);

			
			
			ECTrace.trace( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName, 
				"Returned from OrdPayMthdAccessBeam.commitCopyHelper() ...." );

			ECTrace.trace( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName,
					"This purchase uses the TestPaymentMethod.  OrderState to be returned: " 
					+ OrderConstants.ORDER_AUTH_ALLOWED );
		
			setOrderState( OrderConstants.ORDER_AUTH_ALLOWED );
			
		} catch( javax.ejb.CreateException ex ) {
			throw new ECSystemException( 
					ECMessage._ERR_CREATE_EXCEPTION, CLASSNAME, methodName, ex );
		} catch( javax.ejb.FinderException ex ) {
			throw new ECSystemException( 
					ECMessage._ERR_FINDER_EXCEPTION, CLASSNAME, methodName, ex );
		} catch( javax.naming.NamingException ex ) {
			throw new ECSystemException( 
					ECMessage._ERR_NAMING_EXCEPTION, CLASSNAME, methodName, ex );
		} catch( java.rmi.RemoteException ex ) {
			throw new ECSystemException(
					ECMessage._ERR_REMOTE_EXCEPTION, CLASSNAME, methodName, ex );
		}
		ECTrace.exit( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName );
	}
/**
 * Returns a new string that is free of any spaces and hyphenated characters.
 * If the supplied string is already free of spaces and hyphens the same string
 * may be returned.
 * 
 * @return a string free of spaces and hyphenated characters ('-')
 * @param aCardNumber the card number to be examined and stripped
 */
public static String removeSpaceAndHyphenFromString(String aCardNumber) 
{
	final String strDelim = " -"; 		// Delimiters for the StringTokenizer 
	
	if ( aCardNumber == null ) return null;

	StringTokenizer tokenizer = new StringTokenizer( aCardNumber, strDelim );
	StringBuffer 	strBuf	  = new StringBuffer();

	while ( tokenizer.hasMoreTokens() ) {
		strBuf.append( tokenizer.nextToken() );
	}
	return strBuf.toString();
}
/**
 * Resets the instance variables of the command.
 * The reset() method will reset all instance variables.
 * If the command includes any output values that are accessible via 
 * getters, then make sure those values are no longer needed before calling
 * this method to reset the instance variables of the command.  
 * <p>
 * This is useful if the command instance is to be called multiple times with
 * different command parameters.
 * <p>
 */
public void reset() 
{
	itsOrderId				= null;
	itsTotalAmount  		= null;
	itsCardBrand       		= null;
	itsCardNumber     		= null;
	itsCardExpiryMonth    	= null;
	itsCardExpiryYear    	= null;
	itsPaymentAttributes    = null;
	
}
	public void setAccount (AccountAccessBean abAccount) {iabAccount=abAccount;}
	/**
	 * Set Optional buyerPOId.
	 */
	public void setBuyerPOId(Long n) {
		inBuyerPOId = n;
	}
protected void setCardBrand(String cardBrand)
{
	itsCardBrand = cardBrand;
}
protected void setCardExpiryMonth(Integer cardExpiryMonth)
{
	itsCardExpiryMonth= cardExpiryMonth;
}
protected void setCardExpiryYear(Integer cardExpiryYear)
{
	itsCardExpiryYear= cardExpiryYear;
}
protected void setCardNumber(String cardNumber)
{
	itsCardNumber = removeSpaceAndHyphenFromString( cardNumber );

// ****************************************************************** 	
//  Comment out Trace line for security reason.
//	
//	ECTrace.trace( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, "setCardNumber",
//		"CardNumber: (before) \'" + cardNumber + "\'.  (after) \'" + itsCardNumber 
//		+ "\'" );

}
	/**
	 * Optional error view name passed in by the Caller.
	 */
	public void setErrorViewName (String sEVN) { isErrorViewName = sEVN;}
	/**
	 * The order to be ReApproved.
	 */
	public void setOrder (OrderAccessBean ab) {
		iabOrder = ab;
	}
	protected void setOrderState(String orderState) 
	{ 
		istrOrderState = orderState; 
	}
/**
 * Sets the paymentAttributes property of the command.
 * 
 * @param requestProperties the requestProperties object that contains all
 * the command parameters are to be passed to this command.
 *
 * <P>
 * This command requires the following Payment Attributes:
 * <PRE>
 *       Keyword                         Value
 *     ===========        ====================================
 *     cardBrand          The credit card brand.  Represents the credit card type used.
 *     cardNumber         The credit card number              
 *     cardExpiryMonth    The credit card exipration month  
 *     cardExpiryYear     The credit card exipration year   
 * </PRE>
 * Note that this command stores the credit card brand in the PAYMETHOD column of 
 * the ORDPAYMTHD table and the size of the column is limited to five characters long.
 * 
 */
public void setPaymentAttributes(TypedProperty paymentAttributes)
{
	itsPaymentAttributes = paymentAttributes;
}
/** 
 * Sets the totalAmount property of the command.
 * 
 * @param totalAmount the total price of the Order for which payment
 * is to be processed.
 */
public void setTotalAmount(BigDecimal totalAmount)
{
	itsTotalAmount=totalAmount;
}
/**
 * Checks the validity of customer-supplied parameters.
 */
public void validateParameters() throws ECException
{
	final String methodName = "validateParameters";
	ECTrace.entry( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName );

	// Check for User Input Error and throws ECApplicationException....
	//
	TypedProperty paymentAttributes = getPaymentAttributes();
		
	setCardBrand ( paymentAttributes.getString(ECConstants.EC_CC_TYPE, ECConstants.EC_STR_EMPTY) );
	if ( getCardBrand().trim().length() == 0 )
	{
		TypedProperty nvps  = new TypedProperty();
		String msg = ECMessageHelper.getUserMessage(ECMessage._ERR_PAY_CARD_BRAND_MISSING,
										null, getCommandContext().getLocale() );
		nvps.put( DoPaymentCmd.MESSAGE_FOR_BUYER, msg );
		nvps.put( ECConstants.EC_ERROR_CODE	, PaymentConstants.ERR_CODE_MISSING_PARAM );
		nvps.put( DoPaymentCmd.NVP_FIELD	, ECConstants.EC_CC_TYPE );
		throw new ECApplicationException(
						ECMessage._ERR_PAY_CARD_BRAND_MISSING, CLASSNAME, methodName,
						ERRTASK_NAME, nvps ); 
	}

	setCardNumber( paymentAttributes.getString(ECConstants.EC_CC_NUMBER, ECConstants.EC_STR_EMPTY) );
	if ( getCardNumber().trim().length() == 0 )
	{
		TypedProperty nvps  = new TypedProperty();
		String msg = ECMessageHelper.getUserMessage(ECMessage._ERR_PAY_CARD_NUMBER_MISSING,
										null, getCommandContext().getLocale() );
		nvps.put( DoPaymentCmd.MESSAGE_FOR_BUYER, msg );
		nvps.put( ECConstants.EC_ERROR_CODE	, PaymentConstants.ERR_CODE_MISSING_PARAM   );
		nvps.put( DoPaymentCmd.NVP_FIELD	, ECConstants.EC_CC_NUMBER );
		throw new ECApplicationException(
						ECMessage._ERR_PAY_CARD_NUMBER_MISSING, CLASSNAME, methodName,
						ERRTASK_NAME, nvps );
	}

	try { setCardExpiryMonth( paymentAttributes.getInteger(ECConstants.EC_CCX_MONTH) ); }
	catch ( com.ibm.commerce.exception.ParameterNotFoundException ex ) { 
		TypedProperty nvps  = new TypedProperty();
		String msg = ECMessageHelper.getUserMessage(ECMessage._ERR_PAY_EXPIRY_MONTH_MISSING,
										null, getCommandContext().getLocale() );
		nvps.put( DoPaymentCmd.MESSAGE_FOR_BUYER, msg );
		nvps.put( ECConstants.EC_ERROR_CODE, PaymentConstants.ERR_CODE_MISSING_PARAM   );
		nvps.put( DoPaymentCmd.NVP_FIELD,    ECConstants.EC_CCX_MONTH );
		throw new ECApplicationException( ECMessage._ERR_PAY_EXPIRY_MONTH_MISSING, CLASSNAME, methodName,
						ERRTASK_NAME, nvps );
	}
	catch ( com.ibm.commerce.exception.InvalidParameterValueException ex ) { 
		TypedProperty nvps  = new TypedProperty();
		String msg = ECMessageHelper.getUserMessage(ECMessage._ERR_PAY_EXPIRY_DATE_INVALID,
										null, getCommandContext().getLocale() );
		nvps.put( DoPaymentCmd.MESSAGE_FOR_BUYER, msg );
		nvps.put( ECConstants.EC_ERROR_CODE, PaymentConstants.ERR_CODE_BAD_XDATE       );
		nvps.put( DoPaymentCmd.NVP_FIELD,    ECConstants.EC_CCX_MONTH );
		throw new ECApplicationException( ECMessage._ERR_PAY_EXPIRY_DATE_INVALID, CLASSNAME, methodName,
						ERRTASK_NAME, nvps );
	}

	try { setCardExpiryYear ( paymentAttributes.getInteger(ECConstants.EC_CCX_YEAR) ); }
	catch ( com.ibm.commerce.exception.ParameterNotFoundException ex ) { 
		TypedProperty nvps  = new TypedProperty();
		String msg = ECMessageHelper.getUserMessage(ECMessage._ERR_PAY_EXPIRY_YEAR_MISSING,
										null, getCommandContext().getLocale() );
		nvps.put( DoPaymentCmd.MESSAGE_FOR_BUYER, msg );
		nvps.put( ECConstants.EC_ERROR_CODE, PaymentConstants.ERR_CODE_MISSING_PARAM );
		nvps.put( DoPaymentCmd.NVP_FIELD,    ECConstants.EC_CCX_YEAR );
		throw new ECApplicationException( ECMessage._ERR_PAY_EXPIRY_YEAR_MISSING, CLASSNAME, methodName,
						ERRTASK_NAME, nvps );
	}
	catch ( com.ibm.commerce.exception.InvalidParameterValueException ex ) { 
		TypedProperty nvps  = new TypedProperty();
		String msg = ECMessageHelper.getUserMessage(ECMessage._ERR_PAY_EXPIRY_DATE_INVALID,
										null, getCommandContext().getLocale() );
		nvps.put( DoPaymentCmd.MESSAGE_FOR_BUYER, msg );
		nvps.put( ECConstants.EC_ERROR_CODE, PaymentConstants.ERR_CODE_BAD_XDATE );
		nvps.put( DoPaymentCmd.NVP_FIELD,    ECConstants.EC_CCX_YEAR );
		throw new ECApplicationException( ECMessage._ERR_PAY_EXPIRY_DATE_INVALID, CLASSNAME, methodName,
						ERRTASK_NAME, nvps );
	}

	ECTrace.exit( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName );
}
}
