//********************************************************************
//*-------------------------------------------------------------------
//* Sample Source
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2000, 2002
//*
//* The source code for this program is not published or otherwise
//* divested of its trade secrets, irrespective of what has been
//* deposited with the US Copyright Office.
//*-------------------------------------------------------------------
//*
////////////////////////////////////////////////////////////////////////////////


import com.ibm.commerce.command.*;
import com.ibm.commerce.exception.*;
import com.ibm.commerce.fulfillment.objects.*;
import com.ibm.commerce.contract.objects.*;
import com.ibm.commerce.contract.objimpl.*;
import com.ibm.commerce.common.objects.*;
import com.ibm.commerce.common.objimpl.*;
import com.ibm.commerce.order.objects.*;
import com.ibm.commerce.order.objimpl.*;
import com.ibm.commerce.order.utils.*;
import com.ibm.commerce.price.commands.*;
import com.ibm.commerce.price.utils.*;
import com.ibm.commerce.catalog.objects.*;
import com.ibm.commerce.catalog.objimpl.*;
import com.ibm.commerce.base.objects.*;
import com.ibm.commerce.ras.*;
import com.ibm.commerce.user.objects.*;
import com.ibm.commerce.utils.*;
import com.ibm.commerce.server.*;
import com.ibm.commerce.datatype.*;
import com.ibm.commerce.contract.util.*;
import com.ibm.util.*;
import java.math.*;
import java.rmi.*;
import java.sql.*;
import java.util.*;
import javax.ejb.*;
import javax.naming.*;

/**
 * This <code>RetrievePricesCmd</code> Task Command Implementation
 * retrieves all the qualified offers for one or more catalog entries 
 * from the price lists which are referenced by the entitled contracts. 
 * 
 * <p>
 * If the command is successful in retreiving offers, it will return a list
 * of qualified offers that can provide the offer prices. 
 * <p>
 * If unsuccessful, it wil return an empty list.
 *
 * <p><strong>Behaviour</strong>
 * <p>Obtain all the qualified offers for each catalog entries, each offer 
 * to be retrieved to find the price of a catalog entry must satisfy certain
 * criteria.
 * <p>If no valid offer can be found, the parent product offers will be searched
 * providing the catalog entry and the parent product is not excluded from the 
 * trading agreements.
 */
public class RetrievePricesCmdImpl extends com.ibm.commerce.command.BusinessPolicyCommandImpl implements RetrievePricesCmd {
	
	public static final String COPYRIGHT						= com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;
				
	private static final String CLASS_NAME						= "com.ibm.commerce.price.commands.RetrievePricesCmdImpl";
	private static final String VALIDATE_PARAMETERS				= "validateParameters";
	private static final String PERFORM_EXECUTE					= "performExecute";
	private static final String IS_VALID_OFFER					= "isValidOffer";
	private static final String RETRIEVE_OFFERS_FOR_CATENTRIES	= "retrieveOffersForCatentries";
	private static final String IS_CATENTRY_INCLUDED            = "isCatEntryIncluded";
	private static final String GET_PARENT_CATENTRYIDS      	= "getParentCatEntryIds";	
		
	private static final BigDecimal BIG_DECIMAL_ZERO			= new BigDecimal(0);
	private static final Double DOUBLE_ZERO						= new Double(0);
	
	private String istrCurrency									= null;	
	private Long[] iCatEntryIds 								= null;
	private QuantityAmount[] iQuantityAmounts 			  		= null; 
	private Long[] iOfferIds									= null;
	private Long[] iPriceListIds								= null;
	private Integer inStoreId 									= null;
	private StoreAccessBean iStoreAB = null;
	private CatalogEntryShippingAccessBean iCatalogEntryShippingAB = null;
	private Hashtable[] iItemOffers								= null;
	private Integer[] iPSExclusionList							= null;
	private Integer[] iPSInclusionList							= null;
			
	public RetrievePricesCmdImpl() { super(); }
/**
 * Returns the quantity amount for the input catalog entry.
 * <p>
 * @param aQuantity the quantity to be converted.
 * @param aCatEntryId the input catalog entry.
 * <p>
 * @return The quantity amount for the catalog entry.
 */
private QuantityAmount calculateQuantityAmount(double aQuantity, Long aCatentryId) throws ECSystemException {
	QuantityAmount quantityAmount;
	
	if( iCatalogEntryShippingAB == null ) {
		// PriceList: add aCatentryId as input parm
		iCatalogEntryShippingAB = Helper.getCatalogEntryShippingAB(aCatentryId);
	}

	quantityAmount = Helper.createQuantityAmount(iCatalogEntryShippingAB, aQuantity);

	return quantityAmount;
}
/**
 * Generates a search query to exclude a list of product sets which has been
 * specified in the trading agreements.
 * <p>
 * @return The product set exclusion search query string.
 */
private String createExclusionPredicate() throws ECException {

	String psExclusionPredicate = null;
			
	Integer[] exclusionPsIds = getPSExclusionList();
		
	if(exclusionPsIds != null && exclusionPsIds.length > 0) {
			
		psExclusionPredicate =
			" not in (select prsetcerel.catentry_id from prsetcerel where"
			+ " prsetcerel.productset_id " + PriceCalculationHelper.getInstance().getInPredicate(exclusionPsIds)
			+ " )";
	}		
		
	return psExclusionPredicate;
	
}
/**
 * Generates a search query to include a list of product sets which has been
 * specified in the trading agreements.
 * <p>
 * @return The product set inclusion search query string.
 */
private String createInclusionPredicate() throws ECException {

	String psInclusionPredicate = null;
	
	Integer[] inclusionPsIds = getPSInclusionList();

	if(inclusionPsIds != null && inclusionPsIds.length > 0) {
			
		psInclusionPredicate =
			" in (select prsetcerel.catentry_id from prsetcerel where"
			+ " prsetcerel.productset_id " + PriceCalculationHelper.getInstance().getInPredicate(inclusionPsIds)
			+ " )";
	}	
		
	return psInclusionPredicate;
	
}
/**
 * Returns the ids of the input price lists to be used for price.
 */
private Long[] getCatEntryIds() { return iCatEntryIds; }
/**
 * Returns the currency to be used for the price.
 */
protected String getCurrency() { return istrCurrency; }
/**
 * Returns all the qualified offers retrieved for each of the input items.
 */
public Hashtable[] getItemOffers() { return iItemOffers; }
/**
 * Returns the parent products of a catalog entry.
 * <p>
 * @param catEntryId the id of input catalog entry to obtain the parent product.
 * <p>
 * @return The enumeration of a list of qualified parent products.
 */
private Enumeration getParentCatEntryIds(Long catEntryId) throws ECSystemException {
	
	Enumeration parentProductEnum = null;
	try {
		CatalogEntryAccessBean catEntryAB = new CatalogEntryAccessBean();
		parentProductEnum = catEntryAB.findByChildCatalogEntry(catEntryId);

	} catch (javax.ejb.FinderException ex) {
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, CLASS_NAME, GET_PARENT_CATENTRYIDS, ex);
	} catch (javax.naming.NamingException ex) {
		throw new ECSystemException(ECMessage._ERR_GENERIC, CLASS_NAME, GET_PARENT_CATENTRYIDS, ex);
	} catch (java.rmi.RemoteException ex) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, CLASS_NAME, GET_PARENT_CATENTRYIDS, ex);
	}

	return parentProductEnum;
}
/**
 * Returns the ids of the input price lists to be used for retrieving the offers.
 */
private Long[] getPriceListIds() { return iPriceListIds; }
/**
 * Returns the ids of a list of product sets excluded from the trading agreements.
 */
private Integer[] getPSExclusionList() { return iPSExclusionList; }
/**
 * Returns the ids of a list of product sets included in the trading agreements.
 */
private Integer[] getPSInclusionList() { return iPSInclusionList; }
/**
 * Returns the store access bean.
 *
 * If the store id is not specified then the store id of the command context
 * will be used and the corresponding access bean will be retrieved from the
 * registry.
 */
private com.ibm.commerce.common.objects.StoreAccessBean getStoreAB() {
	if( iStoreAB == null ) {
		Integer storeId = myGetStoreId();
		if( storeId != null ) {
			setStoreAB(WcsApp.storeRegistry.find(storeId));
		}
	}
	return iStoreAB;
}
/**
 * Verifies if the catentry is included in the list of trading agreements.
 * <p>
 * @param aItem the id of the catalog entry.
 * <p>
 * @return true if the catentry is included; false otherwise.
 */
private boolean isCatentryIncluded(Long aItem) throws ECException {

	Integer[] psExclusion = getPSExclusionList();
	Integer[] psInclusion = getPSInclusionList();

	// PriceList: exclusion takes precedence on inclusion
	if(psExclusion != null && psExclusion.length > 0) {
		for(int i=0; i<psExclusion.length; i++) {
			try {
				ProductSetCeRelAccessBean productSetRelAB = new ProductSetCeRelAccessBean();
				productSetRelAB.setInitKey_catalogEntryId(aItem.toString());
				productSetRelAB.setInitKey_productSetId(psExclusion[i].toString());
				productSetRelAB.getEJBRef();
				return false;
			}
			catch(ObjectNotFoundException e) {}
			catch(Exception e) {
				throw new ECSystemException(ECMessage._ERR_GENERIC, CLASS_NAME, IS_CATENTRY_INCLUDED, new Object[] { e.getMessage() }, e);
			}
		}
	}	

	if(psInclusion != null && psInclusion.length > 0) {

		for(int i=0; i<psInclusion.length; i++) {
			try {
				ProductSetCeRelAccessBean productSetRelAB = new ProductSetCeRelAccessBean();
				productSetRelAB.setInitKey_catalogEntryId(aItem.toString());
				productSetRelAB.setInitKey_productSetId(psInclusion[i].toString());
				productSetRelAB.getEJBRef();
				return true;
			}
			catch(ObjectNotFoundException e) {}	
			catch(Exception e) {
				throw new ECSystemException(ECMessage._ERR_GENERIC, CLASS_NAME, IS_CATENTRY_INCLUDED, new Object[] { e.getMessage() }, e);
			}
		}
		return false;
	}	
		
	return true;
	
}
/**
 * Verifies if the offer is valid given the requested quantity amount.
 * <p>
 * @param aOfferAB the offer access bean to be validated.
 * @param aQuantity the quantity amount to be checked against the offer.
 * <p>
 * @return true if the offer is valid; false otherwise.
 */
private boolean isValidOffer(OfferAccessBean aOfferAB, QuantityAmount aQuantity) throws ECException {
		
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, IS_VALID_OFFER);

	boolean isValid = true;
	
	try {
		// PriceList: comment out
		// minor workaround - need a new finder method for a particular offer 
		//if( iOfferId != null ) {
		//	isValid = iOfferId.equals( anOffer.getOfferIdInEJBType() );
		//}

		// PriceList
		double quantityValue = getQuantityValue(aQuantity);
        boolean conversionValid = true;
		//if (isValid) {
			double minimumQuantity;
			double maximumQuantity;

			Double minQuantity = aOfferAB.getMinimumQuantityInEJBType();
			Double maxQuantity = aOfferAB.getMaximumQuantityInEJBType();

			// process minimum amount	
			if (minQuantity == null) {
				minimumQuantity = 0.0;
			} else {
				minimumQuantity = minQuantity.doubleValue();
			}

			// process maximum amount
			if (maxQuantity == null) {
				maximumQuantity = Float.POSITIVE_INFINITY;
			} else {
				maximumQuantity = maxQuantity.doubleValue();
			}

			String qtyUnit = aOfferAB.getQuantityUnitId();
			// optimization: If offer.quantityUnit = quantity unit of requested quantity,
            // don't do conversions to catalog entry shipping record.
            // This line effectively only checks that the 2 quantities are null.  The chances of both 
            // quantity values pointing to the same memory location are slim.  If the 2 quantity 
            // values are equal() and not null, the path in the else portion will be short and will 
            // avoid the extra SQL for CATENTSHIP.
            if (qtyUnit == getQuantityUnit(aQuantity)) {
                // don't do conversions
            } else {
				if( qtyUnit == null ) {
					// convert maximum and minimum to nominal quantity in the catalog entry
					QuantityAmount minQuantityAmount = calculateQuantityAmount( minimumQuantity, aOfferAB.getCatalogEntryReferenceNumberInEJBType() );
					QuantityAmount maxQuantityAmount = calculateQuantityAmount( maximumQuantity, aOfferAB.getCatalogEntryReferenceNumberInEJBType());

					// set values to new calculated amounts
					qtyUnit = minQuantityAmount.getQuantityUnit();
					minimumQuantity = minQuantityAmount.getValue();
					maximumQuantity = maxQuantityAmount.getValue();
				}

				// convert to the expected quantity unit
				// get the quantity manager
				QuantityManager quantityManager = QuantityManager.getInstance();

				// PriceList
				// get converted quantity amount
				QuantityAmount convertedQuantityAmount = quantityManager.convert(getQuantityAmount(aQuantity,aOfferAB.getCatalogEntryReferenceNumberInEJBType()), qtyUnit);

				if( convertedQuantityAmount == null ) {
	                conversionValid = false;
                } else {
                	quantityValue = convertedQuantityAmount.getValue();
                }
            }

            ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, IS_VALID_OFFER, "valid conversion: " + conversionValid + 
	            																		  "	UOM: " + qtyUnit);
            ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, IS_VALID_OFFER, "input quantity: " + quantityValue + 
	                             														  " min quantity: " + minimumQuantity +
	                             														  " max quantity: " + maximumQuantity);
			
            // if no valid conversion or the quantity does not qualify for this offer
			if (!conversionValid
                || (minimumQuantity > quantityValue)
                || (maximumQuantity < quantityValue)) {
                isValid = false;
       		}
            		
		//}
	} catch (javax.ejb.CreateException ex) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, CLASS_NAME, IS_VALID_OFFER, ex);
	} catch (javax.ejb.FinderException ex) {
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, CLASS_NAME, IS_VALID_OFFER, ex);
	} catch (javax.naming.NamingException ex) {
		throw new ECSystemException(ECMessage._ERR_GENERIC, CLASS_NAME, IS_VALID_OFFER, ex);
	} catch (java.rmi.RemoteException ex) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, CLASS_NAME, IS_VALID_OFFER, ex);
	}

	ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, IS_VALID_OFFER, "valid offer = " + isValid);
	
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, IS_VALID_OFFER);

	return isValid;
}
/**
 * Resolves the store id. If the store id is not specified then the store id of
 * the command context will be used.
 * <p>
 * @return The store id to be used.
 */
private java.lang.Integer myGetStoreId() {
	if( inStoreId == null ) inStoreId = getStoreId();
	return inStoreId;
}
/**
 * Executes main business logic of the command.
 */
public void performExecute() throws ECException {
		
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, PERFORM_EXECUTE);
		
	//checkParameters();
					
	try {

		Integer storeId = getStoreAB().getStoreEntityIdInEJBType();			
					
		iItemOffers = retrieveOffersForCatentries(storeId);
		
			
	} catch (javax.ejb.CreateException ex) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, CLASS_NAME, PERFORM_EXECUTE, ex);
	} catch (javax.ejb.FinderException ex) {
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, CLASS_NAME, PERFORM_EXECUTE, ex);
	} catch (javax.naming.NamingException ex) {
		throw new ECSystemException(ECMessage._ERR_GENERIC, CLASS_NAME, PERFORM_EXECUTE, ex);
	} catch (java.rmi.RemoteException ex) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, CLASS_NAME, PERFORM_EXECUTE, ex);
	}

				
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, PERFORM_EXECUTE);

}
/**
 * Obtains all the qualified offers for the input catalog entries and their
 * parent products. 
 * Each offer to be retrieved must satisfy certain criteria.
 * <p>The inclusion and exclusion product sets are taken into account when
 * searching the offers.
 * <p>
 * @param storeId The id of the store for search.
 * <p>
 * @return A list of qualified offers for each of the input catalog entries.
 */
protected Hashtable[] retrieveOffersForCatentries(Integer storeId) throws ECException {
	
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, RETRIEVE_OFFERS_FOR_CATENTRIES);
	
	boolean timestampFlag = false;
	Timestamp timestamp = null;
	Hashtable[] itemOffers = new Hashtable[iCatEntryIds.length];

	try {
			// PriceList: set up the input information for looking up offers
			Vector productItemVec = new Vector();
			Vector parentProductVec = new Vector();
			Hashtable childParentProductHash = new Hashtable();
			
		    for(int j=0; j<iCatEntryIds.length; j++) {

			    // PriceList: to check if the input item is included or excluded
			    //if(!includedInTradings(catEntryId,iTradingABs)) {
				//    PriceCalculationHelper.handlePriceNotFoundException(iItemPriceInfo[j], getCurrency(), CLASS_NAME);
			    //}

			    if(!productItemVec.contains(iCatEntryIds[j]))
			    	productItemVec.addElement(iCatEntryIds[j]);

			    // PriceList?: the following code can be eliminated if the find method also search
			    //             on the parent product  	
			    Enumeration parentProductsEnum = getParentCatEntryIds(iCatEntryIds[j]);
				if(parentProductsEnum.hasMoreElements()) {
					Vector productVec = (Vector)childParentProductHash.get(iCatEntryIds[j]);
					if(productVec == null) {
						productVec = new Vector(1);
						childParentProductHash.put(iCatEntryIds[j], productVec);
					}	
										
			 	 	while( parentProductsEnum.hasMoreElements() ) {
						Long productId = ((CatalogEntryAccessBean) parentProductsEnum.nextElement()).getCatalogEntryReferenceNumberInEJBType();
						if(!productVec.contains(productId))
							productVec.addElement(productId);
						if(!productItemVec.contains(productId))	
							productItemVec.addElement(productId);
				
					}
				}  	
			}

		    Long[] productItemIds = (Long[])CalculationCmdHelper.toArray(productItemVec, Long.class);
		    
		   	String exclusionPredicate = createExclusionPredicate();
		   	String inclusionPredicate = createInclusionPredicate();		
	
		   	 
			for(int x=0; x<productItemIds.length; x++) {
				ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, RETRIEVE_OFFERS_FOR_CATENTRIES, 
					"retrieving offers for catentry: " + productItemIds[x].toString() + " in store: " + storeId.toString());
			}

			for(int y=0; y<iPriceListIds.length; y++) {
				ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, RETRIEVE_OFFERS_FOR_CATENTRIES, 
					"retrieving offers from price list: " + iPriceListIds[y].toString());
			}

			if(iOfferIds != null) {
				for(int z=0; z<iOfferIds.length; z++) {
					ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, RETRIEVE_OFFERS_FOR_CATENTRIES, 
						"retrieving offers from offer: " + iOfferIds[z].toString());
				}
			}


			Enumeration offersEnum = null;
			
			//offersEnum = 
			//	new OfferAccessBean().findEffectiveOffersByTimestampCatentriesOffersStorePriceListsAndMemberGroups(
			//	timestampFlag, timestamp, iCatEntryIds, iOfferIds, storeId, iPriceListIds, iMemberGroupIds);
			
			offersEnum =
		   		new OfferAccessBean().findEffectiveOffersByTimestampCatentriesPredicateOffersStoreAndPriceLists(
			 		timestampFlag, timestamp, productItemIds, iOfferIds, storeId, iPriceListIds, exclusionPredicate, 
			   		inclusionPredicate);
						
			if (offersEnum != null) {
				//double maxPrecedence = -1;
				//long containerId = -1000;
				OfferAccessBean tmp;

				Long prevCatEntry = null;
				QuantityAmount qty = null;
				int itemIndex = 0;

				Vector offerABList = new Vector();
				
				while (offersEnum.hasMoreElements()) {
					OfferAccessBean offerAB = (OfferAccessBean)offersEnum.nextElement();
					offerABList.addElement(offerAB);
				}	

				// PriceList: process for each catalog entry
				for(int i=0; i<iCatEntryIds.length; i++) {
					// Double highestPrcList = new Double(Double.MIN_VALUE); 
				
					Hashtable offerInfoHash = new Hashtable();
					itemOffers[i] = offerInfoHash;
					 
					for(int j=0; j<offerABList.size(); j++) {
						OfferAccessBean offerAB = (OfferAccessBean)offerABList.elementAt(j);
						Long offerCat = offerAB.getCatalogEntryReferenceNumberInEJBType();
						// PriceList: Do I qualify for this offer 
						if((iCatEntryIds[i].equals(offerCat)) && (isValidOffer(offerAB, iQuantityAmounts[i]))) {
							//String priceListId = offerAB.getTradePosContainerId();
							//TradingPositionContainerAccessBean priceListAB = new TradingPositionContainerAccessBean();
							//priceListAB.setInitKey_tradingPositionContainerId(priceListId);
							// PriceList: need to create getPrecedence from TPC
							//if(priceListAB.getPrecedence > highestPrcList)
							//	highestPrcList = priceListAB.getPrecedence;
								
							//QualifyingOfferInfo offerInfo = new QualifyingOfferInfo();
							//offerInfo.setOffer(offerAB);
							//offerInfoHash.put(offerAB.getOfferIdInEJBType(), offerInfo);
							offerInfoHash.put(offerAB.getOfferIdInEJBType(), offerAB);
						}	
							
					}	
					
					/*
					Enumeration offerHashEnum = offerInfoHash.elements();
					while(offerHashEnum.hasMoreElements()) {
						QualifyingOfferInfo offerInfo = (QualifyingOfferInfo)offerHashEnum.nextElement();
						OfferAccessBean offerAB = offerInfo.getOffer();
						String priceListId = offerAB.getTradePosContainerId();
							TradingPositionContainerAccessBean priceListAB = new TradingPositionContainerAccessBean();
							priceListAB.setInitKey_tradingPositionContainerId(priceListId);
							if(priceListAB.getPrecedence != highestPrcLst)
								offerInfoHash.remove(offerAB.getOfferIdInEJBType());
					}
					*/	

					if(offerInfoHash == null || offerInfoHash.size() <= 0) {
						// can't find a price for the item, get the parent products
						if(offerABList.size() > 0 && childParentProductHash.size() > 0) {
							Vector productVec = (Vector)childParentProductHash.get(iCatEntryIds[i]);
							if(productVec != null && productVec.size() > 0) {
								// continue only if the input item is included in product sets
								//if(includedInTradings(catEntry,iTradingABs)) {
								if(isCatentryIncluded(iCatEntryIds[i])) {
									for(int k=0; k<offerABList.size(); k++) {
										OfferAccessBean offerAB = (OfferAccessBean)offerABList.elementAt(k);
										Long offerCat = offerAB.getCatalogEntryReferenceNumberInEJBType();
										// PriceList: Do I qualify for this offer 
										if((productVec.contains(offerCat)) && (isValidOffer(offerAB, iQuantityAmounts[i]))) {
											//QualifyingOfferInfo offerInfo = new QualifyingOfferInfo();
											//offerInfo.setOffer(offerAB);
											//offerInfoHash.put(offerAB.getOfferIdInEJBType(), offerInfo);
											offerInfoHash.put(offerAB.getOfferIdInEJBType(), offerAB);
											
										}
									}
								}	
								
							}		
						}	
					}			
							
				}
			}		
					
	} catch (javax.ejb.CreateException ex) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, CLASS_NAME, RETRIEVE_OFFERS_FOR_CATENTRIES, ex);
	} catch (javax.ejb.FinderException ex) {
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, CLASS_NAME, RETRIEVE_OFFERS_FOR_CATENTRIES, ex);
	} catch (javax.naming.NamingException ex) {
		throw new ECSystemException(ECMessage._ERR_GENERIC, CLASS_NAME, RETRIEVE_OFFERS_FOR_CATENTRIES, ex);
	} catch (java.rmi.RemoteException ex) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, CLASS_NAME, RETRIEVE_OFFERS_FOR_CATENTRIES, ex);
	}

	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, RETRIEVE_OFFERS_FOR_CATENTRIES);

	return itemOffers;
}
/**
 * Sets the ids of the input catentries whose prices are to be calculated.
 */
public void setCatEntryIds(Long[] aCatEntryIds) {
	iCatEntryIds = aCatEntryIds;
}
/**
 * Sets the currency to be used for finding the price (Optional).
 */
public void setCurrency(String astrCurrency) { istrCurrency = astrCurrency; }
/**
 * Sets the qualified offers retrieved for each of the items.
 */
public void setItemOffers(Hashtable[] aItemOffers) { 
	iItemOffers = aItemOffers;
}
/**
 * Sets the ids of the input offers to be used for retrieving the offer prices (Optional).
 */
public void setOfferIds(Long[] aOfferIds) {
	iOfferIds = aOfferIds;
}
/**
 * Sets the ids of the input price lists to be used for retrieving the offers.
 */
public void setPriceListIds(Long[] aPriceListIds) { iPriceListIds = aPriceListIds; }
/**
 * Sets the list of product sets excluded from the trading agreements.
 */
public void setPSExclusionList(Integer[] aPSExclusionList) { iPSExclusionList = aPSExclusionList; }
/**
 * Sets the list of product sets included in the trading agreements.
 */
public void setPSInclusionList(Integer[] aPSInclusionList) { iPSInclusionList = aPSInclusionList; }
/**
 * Sets the quantities of the catentries to be used for calculating the prices.
 */
public void setQuantities(QuantityAmount[] aQuantities) {
	iQuantityAmounts = aQuantities;
}
/**
 * Sets the request properties for the policy.
 *
 * @param requestProperties the request properties for the policy
 */
public void setRequestProperties(com.ibm.commerce.datatype.TypedProperty requestProperties) {
	this.requestProperties = requestProperties;
}
/**
 * Sets the store Access Bean.
 *
 * @param aStoreAB the access bean of the store. 
 */
private void setStoreAB(StoreAccessBean aStoreAB) {
	iStoreAB = aStoreAB;
}
/**
 * Sets the id of the input store to be used for calculating the price (Optional).
 */
public void setStoreId(java.lang.Integer aStoreId) {
	inStoreId = aStoreId;
}
/**
 * Checks mandatory parameters.
 */
public void validateParameters() throws ECException {
	
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, VALIDATE_PARAMETERS);

	// check the trading agreements
	//if (getTradingABs() == null) {
	//	throw new ECApplicationException(ECMessage._ERR_BAD_MISSING_CMD_PARAMETER, CLASS_NAME, VALIDATE_PARAMETERS , new Object[] {PriceCalculationConstants.TRADINGAGREEMENTS});
	//}
	
	// check the price lists
	if (getPriceListIds() == null) {
		throw new ECApplicationException(ECMessage._ERR_BAD_MISSING_CMD_PARAMETER, CLASS_NAME, VALIDATE_PARAMETERS , new Object[] {PriceCalculationConstants.PRICELISTS});
	}

	// check the items and other related information
	if (getCatEntryIds() == null) { 
		throw new ECApplicationException(ECMessage._ERR_BAD_MISSING_CMD_PARAMETER, CLASS_NAME, VALIDATE_PARAMETERS, new Object[] {PriceCalculationConstants.ITEMINFO});
	}

	// check the currency
	if (getCurrency() == null) {
		throw new ECApplicationException(ECMessage._ERR_BAD_MISSING_CMD_PARAMETER, CLASS_NAME, VALIDATE_PARAMETERS, new Object[] {ECConstants.EC_CURRENCY_ID});
	}
		
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, VALIDATE_PARAMETERS);
	
}

/**
 * Returns the quantity amount of the input catalog entry for price.
 * If the quantity amount is not specified then the nominal quantity
 * (rounded up to the nearest multiple as specified in the quantityMultiple attribute)
 * of the catalog entry is assumed to be the requested quantity.
 */
protected QuantityAmount getQuantityAmount(QuantityAmount aQuantity, Long aCatEntryId) throws ECSystemException {
	// if the quantity was not supplied, use the rounded quantity from the catalog entry shipping record
 	if (aQuantity == null) 
 	 	return getRoundedQuantityAmount(1.0, aCatEntryId);
 	else
		return aQuantity;
}

/**
 * Returns the quantity unit only.
 */
private String getQuantityUnit(QuantityAmount aQuantityAmount) {
	String result = null;
	
	// if the quantity was supplied
	if (aQuantityAmount != null) {
		result = aQuantityAmount.getQuantityUnit();
	}

	//
	return result;
}

/**
 * Returns the quantity value only.
 */
private double getQuantityValue(QuantityAmount aQuantityAmount) {
	double result = 1.0;

	// if the quantity was supplied
	if (aQuantityAmount != null) {
		result = aQuantityAmount.getValue();
	}

	//
	return result;
}

/**
 * Returns the quantity amount that has been rounded up for the input catalog entry.
 * The amount is rounded up to the nearest multiple as specified in the
 * quantityMultiple attribute of the catalog entry
 * <p>
 * @param aQuantity the quantity to be converted.
 * @param aCatEntryId the input catalog entry.
 * <p>
 * @return The rounded up quantity amount for the catalog entry.
 */
private QuantityAmount getRoundedQuantityAmount(double aQuantity, Long aCatEntryId) throws ECSystemException {
	QuantityAmount quantityAmount;
	
	if( iCatalogEntryShippingAB == null ) {
		iCatalogEntryShippingAB = Helper.getCatalogEntryShippingAB(aCatEntryId);
	}

	quantityAmount = Helper.createRoundedQuantityAmount(iCatalogEntryShippingAB, aQuantity);

	return quantityAmount;
}
}
