//********************************************************************
//*-------------------------------------------------------------------
//* Sample Source
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2000, 2002
//*
//* The source code for this program is not published or otherwise
//* divested of its trade secrets, irrespective of what has been
//* deposited with the US Copyright Office.
//*-------------------------------------------------------------------
//*
////////////////////////////////////////////////////////////////////////////////

import com.ibm.commerce.returns.commands.*;

import com.ibm.commerce.inventory.objects.*;
import java.sql.*;
import com.ibm.commerce.exception.*;
import com.ibm.commerce.ras.*;
import com.ibm.commerce.command.*;
import java.util.*;
import javax.naming.*;
import java.rmi.*;
import javax.ejb.*;
import com.ibm.commerce.ordermanagement.objects.*;
import com.ibm.commerce.contract.helper.*;
import com.ibm.commerce.order.objects.*;
 
/**
 * Overall command used for determining if a return mechandise authorization (RMA) item may be approved programatically.
 * Calls AutoApproveReturnItemStandardChecksCmd and AutoApproveReturnItemPolicyCmd for
 * approval logic.
 *
 * Gets a list of all auto-approve deny reasons for the current store, calls
 * AutoApproveReturnItemStandardChecksCmd passing in list of possible deny reasons,
 * calls AutoApproveReturnItemPolicyCmd passing in list of possible deny reasons,
 * and based on list of actual deny reasons passed back, calculates new approval status
 * and logs deny reasons against the RMA item.
 */
public class AutoApproveReturnItemCmdImpl extends com.ibm.commerce.command.TaskCommandImpl implements AutoApproveReturnItemCmd {
	private static final String COPYRIGHT = com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;
	private final static java.lang.String className = AutoApproveReturnItemCmdImpl.class.getName();
	private RMAItemAccessBean RMAItemAB = null;

	private ArrayList denyReasonABListForThisStore = null;
	private RMAAccessBean rmaAB = null;
	private List RMAItemDenyReasonList = new ArrayList(0);
/**
 * AutoApproveReturnItemCmdImpl constructor.
 */
public AutoApproveReturnItemCmdImpl() {
	super();
}
/**	Calculate whether RMA item can be approved programatically. Calls AutoApproveReturnItemStandardChecksCmd
 * and AutoApproveReturnItemPolicyCmd, passing in a list of all possible deny reasons by code for a store.
 * Converts returned list of deny reason applicable to the RMA item, from the generic codes to ids specific
 * to store.  Determines new RMA item status, and calls ApproveReturnItemCmd to set status and the deny reasons
 * if any.
 *
 * If there is no auto-approval policy command applicable to the RMA, then a denial reason of
 * ReturnConstants.DENY_REASON_NO_APPROVAL_POLICY will be applied to the RMA item.
 */
 /* AW 11/21/01 Changed scoping from public to protected
 */
protected void autoApproveRMAItem()
	throws FinderException, RemoteException, NamingException, CreateException, ECException {

	final String methodName = "autoApproveRMAItem";
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, className, methodName);
	ECMessageLog.out(ECMessage._STA_WRITING_ORDERITEM, className, methodName);

	/* Get list of Deny reasons that are applicable - need to pass to the individual checks */
	loadDenyReasons();

	/* Call Standard Checks */
	AutoApproveReturnItemStandardChecksCmd standardCheckApi =
		(AutoApproveReturnItemStandardChecksCmd) CommandFactory.createCommand(
			AutoApproveReturnItemStandardChecksCmd.NAME,
			commandContext.getStoreId());
	standardCheckApi.setCommandContext(commandContext);
	standardCheckApi.setRMAItemAB(this.RMAItemAB);
	standardCheckApi.setRMAItemDenyReasonList(this.RMAItemDenyReasonList);
	standardCheckApi.execute();
	this.RMAItemDenyReasonList = standardCheckApi.getRMAItemDenyReasonList();

	/* Call Policy Based Checks */
	Long tradingId = rmaAB.getTradingIdInEJBType();

	Long memberId = calcMemberIdForPolicy();
	
	Long policyId =
		MiscContract.getOnlyApplicablePolicyId(
			tradingId,
			ECContractConstants.EC_ELE_RETURN_TC_RETURN_CHARGE,
			ECContractConstants.EC_POLICY_TYPE_RETURNAPPROVAL,
			commandContext,
			memberId);

	/* There must be a policy.  To approve everything, then don't have a days argument in policy */
	if (policyId == null) {
		this.RMAItemDenyReasonList.add(ReturnConstants.DENY_REASON_NO_APPROVAL_POLICY);
	} else {
		AutoApproveReturnItemPolicyCmd autoApprovePolicyCmd =
			(AutoApproveReturnItemPolicyCmd) CommandFactory.createBusinessPolicyCommand(
				policyId,
				AutoApproveReturnItemPolicyCmd.NAME);
		autoApprovePolicyCmd.setCommandContext(commandContext);
		autoApprovePolicyCmd.setRMAItemAB(this.RMAItemAB);
		autoApprovePolicyCmd.setRMAItemDenyReasonList(this.RMAItemDenyReasonList);
		autoApprovePolicyCmd.execute();
	}

	/* Set the status in RMA item & record deny reasons if appropriate */
	if (RMAItemDenyReasonList.size() < 1)
		Misc.approveRMAItem(RMAItemAB, ApproveReturnItemCmd.cAutoApproval, getCommandContext());
	else {
		Vector RMAItemDenyReasonIdVector = convertDenyReasons(RMAItemDenyReasonList);
		Misc.approveRMAItem(RMAItemAB, ApproveReturnItemCmd.cPendingApproval, getCommandContext(), RMAItemDenyReasonIdVector);
	}

	Timestamp now = com.ibm.commerce.utils.TimestampHelper.systemCurrentTimestamp();
	RMAItemAB.setLastUpdate(now);

}
/**
 * Calls methods in MiscContract to determine who's member id should be used
 * for determining the policies applicable to the RMA.
 * @return java.lang.Long
 */
private Long calcMemberIdForPolicy() throws ECException {
	final String methodName = "calcMemberIdForPolicy";
	Long memberId = null;

	try {
		Long orderItemId = RMAItemAB.getOrderItemsIdInEJBType();

		if (orderItemId != null) {
			OrderItemAccessBean orderItemAB = new OrderItemAccessBean();
			orderItemAB.setInitKey_orderItemId(orderItemId.toString());

			memberId = orderItemAB.getMemberIdInEJBType();
		} else {
			// this isn't an orderitem based return item
			memberId = MiscContract.getMemberIfForPolicy(rmaAB);
		}
	} catch (javax.ejb.CreateException ex) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, className, methodName, ex);
	} catch (javax.ejb.FinderException ex) {
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, className, methodName, ex);
	} catch (javax.naming.NamingException ex) {
		throw new ECSystemException(ECMessage._ERR_GENERIC, className, methodName, ex);
	} catch (java.rmi.RemoteException ex) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, className, methodName, ex);
	}

	return memberId;

}
/**	Convert List of Deny Reason codes to DenyReason_ids.  Looks up appropriate deny reason id based
 *	on generic deny reason code used across stores.
 * @param RMAItemDenyReasonList List of deny reason codes
 * @return Deny reason Ids
 */
  /* AW 11/21/01 Changed scoping from public to protected
 */
protected Vector convertDenyReasons(List RMAItemDenyReasonList)
    throws FinderException, RemoteException, NamingException, CreateException, ECException {

    final String methodName = "logDenyReason";
    ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, className, methodName);
    ECMessageLog.out(ECMessage._STA_WRITING_ORDERITEM, className, methodName);

    Vector RMAItemDenyReasonIdVector = new Vector(0);
    Integer denyId = null;
    for (int i = 0; i < RMAItemDenyReasonList.size(); i++) {
        /* Log the RMA item denial with the right deny reason id for the store*/
        denyId = determineDenyReason((String) RMAItemDenyReasonList.get(i), denyReasonABListForThisStore);

        if (denyId != null) {
            RMAItemDenyReasonIdVector.add(denyId);
        }
        /* If we didn't find a match the it's a setup error (should never ever get here) */
        else {
            throw new ECSystemException(
                ECMessage._ERR_GENERIC,
                className,
                methodName,
                "Return Deny Reasons not properly loaded.");
        }
    }

    return RMAItemDenyReasonIdVector;
}
/**	Looks up appropriate deny reason id based on generic deny reason code used across stores.
 * @return Deny reason Id
 * @param denyCode Deny reason code
 * @param DenyReasonABList List of possible deny reason Ids and codes for the store.
 */
 /* AW 11/21/01 Changed scoping from public to private
 */
private static Integer determineDenyReason(String denyCode, List DenyReasonABList) 
	throws javax.ejb.FinderException, 
				java.rmi.RemoteException, 
				javax.naming.NamingException, 
				javax.ejb.CreateException, ECException {

	final String methodName = "determineDenyReason";
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER,className, methodName);
	ECMessageLog.out(ECMessage._STA_WRITING_ORDERITEM, className, methodName);

	ReturnDenyReasonAccessBean abDenyReason = null;
	Integer denyId = null;

	/* Loop through all the Deny Reasons until we hit the code required.
	 If there are duplicates between the store and the store group, pick the one for the store.
	 All the ones specific to the store will come before the store group, so we 
	 can just pick the first match */
	for (int i = 0; i < DenyReasonABList.size(); i++) {
		abDenyReason = (ReturnDenyReasonAccessBean) DenyReasonABList.get(i);
		if (abDenyReason.getCode().equals(denyCode)) {
			denyId = abDenyReason.getRtnDnyRsnIdInEJBType();
			break;
		}
	} 

	return denyId;
}
/**
 * Converts an Enumeration to an ArrayList
 * @return Converted ArrayList
 * @param enum java.util.Enumeration
 */
  /* AW 11/21/01 Changed scoping from public to private
 */
private static ArrayList enumerationToArrayList(Enumeration enum) {
  ArrayList arrayList = new ArrayList(0);
  while (enum.hasMoreElements()) {
		arrayList.add(enum.nextElement());
  }
  return arrayList;
}
/**	
 * Gets list of deny reasons, code and id, applicable to current store.
 */
  /* AW 11/21/01 Changed scoping from public to protected
 */
protected void loadDenyReasons()
	throws FinderException, RemoteException, NamingException, CreateException, ECException {

	final String methodName = "getDenyReasons";
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, className, methodName);
	ECMessageLog.out(ECMessage._STA_WRITING_ORDERITEM, className, methodName);

	ReturnDenyReasonAccessBean abDenyReason = null;
	Integer denyId = null;

	/* I'm guessing that getting the small list of deny reasons from the database once
	 and looping through several times is still faster than multiple db calls. */
	/* Get list of Deny Reason for store if not already done */
	if (this.denyReasonABListForThisStore == null) {
		abDenyReason = new ReturnDenyReasonAccessBean();
		Integer test = null;
		Enumeration enDenyReason = abDenyReason.findByStoreIdorStoreGroupIdOrderDescByStoreEnt(commandContext.getStoreId(), commandContext.getStore().getStoreGroupIdInEJBType());
		denyReasonABListForThisStore = enumerationToArrayList(enDenyReason);
	}

}
/**
 * Execute the command.
 */
public void performExecute() throws ECException {
	final String methodName = "performExecute";
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, className, methodName);

	super.performExecute();

	try {
		autoApproveRMAItem();
	}
	catch (NamingException e) {
		throw new ECSystemException(ECMessage._ERR_GENERIC,className,methodName,
																ECMessageHelper.generateMsgParms(e.getMessage()),e);
	}
	catch (RemoteException e) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION,className,methodName,
																ECMessageHelper.generateMsgParms(e.getMessage()),e);
	}
	catch (CreateException e) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION,className,methodName,
																ECMessageHelper.generateMsgParms(e.getMessage()),e);
	}
	catch (FinderException e) {
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION,className,methodName,
																ECMessageHelper.generateMsgParms(e.getMessage()),e);
	}

}
public void reset() {
	RMAItemAB = null;
	denyReasonABListForThisStore = null;
	RMAItemDenyReasonList = new ArrayList(0);
}
/**
 * Set the RMA being accessed.
 * @param rmaAB the RMA access bean.
 */
public void setRMAAB(RMAAccessBean rmaAB) {
	this.rmaAB = rmaAB;
}
/**
 * Set the RMA item being accessed.
 * @param RMAitemAB the RMA item access bean.
 */
public void setRMAItemAB(RMAItemAccessBean RMAItemAB) {
	this.RMAItemAB = RMAItemAB;
}
public void validateParameters() throws ECException {
    final String methodName = "validateParameters";
    ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, className, methodName);

    super.validateParameters();

    /* check the RMA item*/
    if (RMAItemAB == null) {
        throw new ECApplicationException(
            ECMessage._ERR_BAD_MISSING_CMD_PARAMETER,
            className,
            methodName,
            new Object[] { ReturnConstants.EC_RMA_ITEM_ID });
    }

    if (rmaAB == null) {
        throw new ECApplicationException(
            ECMessage._ERR_BAD_MISSING_CMD_PARAMETER,
            className,
            methodName,
            new Object[] { ReturnConstants.EC_RMA_ID });
    }

    /* check the trading ID */
    try {
        if (rmaAB.getTradingIdInEJBType() == null) {
            throw new ECSystemException(
                ECMessage._ERR_BAD_MISSING_CMD_PARAMETER,
                className,
                methodName,
                new Object[] { Misc.EC_TRADING_ID });
        }
    } catch (NamingException e) {
        throw new ECSystemException(
            ECMessage._ERR_GENERIC,
            className,
            methodName,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e);
    } catch (RemoteException e) {
        throw new ECSystemException(
            ECMessage._ERR_REMOTE_EXCEPTION,
            className,
            methodName,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e);
    } catch (CreateException e) {
        throw new ECSystemException(
            ECMessage._ERR_CREATE_EXCEPTION,
            className,
            methodName,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e);
    } catch (FinderException e) {
        throw new ECSystemException(
            ECMessage._ERR_FINDER_EXCEPTION,
            className,
            methodName,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e);
    }

    ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, className, methodName);
}
}
