//********************************************************************
//*-------------------------------------------------------------------
//* Sample Source
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2000, 2002
//*
//* The source code for this program is not published or otherwise
//* divested of its trade secrets, irrespective of what has been
//* deposited with the US Copyright Office.
//*-------------------------------------------------------------------
//*
////////////////////////////////////////////////////////////////////////////////

import com.ibm.commerce.tools.campaigns.*;

import com.blazesoft.server.deploy.rules.NdDeployRulesServiceConfig;
import com.ibm.commerce.catalog.objects.CatalogAccessBean;
import com.ibm.commerce.catalog.objects.CatalogEntryAccessBean;
import com.ibm.commerce.catalog.objects.CatalogGroupAccessBean;
import com.ibm.commerce.command.CommandFactory;
import com.ibm.commerce.command.TaskCommandImpl;
import com.ibm.commerce.common.objects.StoreAccessBean;
import com.ibm.commerce.condition.Condition;
import com.ibm.commerce.condition.ConditionConstants;
import com.ibm.commerce.condition.ConditionUtil;
import com.ibm.commerce.condition.OpenCondition;
import com.ibm.commerce.condition.SimpleCondition;
import com.ibm.commerce.copyright.IBMCopyright;
import com.ibm.commerce.couponpromotion.objects.CouponPromotionAccessBean;
import com.ibm.commerce.exception.ECApplicationException;
import com.ibm.commerce.exception.ECException;
import com.ibm.commerce.fulfillment.objects.CalculationCodeAccessBean;
import com.ibm.commerce.fulfillment.objects.CalculationRuleAccessBean;
import com.ibm.commerce.fulfillment.objects.CalculationRuleMemberGroupRelationshipAccessBean;
import com.ibm.commerce.marketing.beans.EMarketingSpot;
import com.ibm.commerce.marketingcenter.campaign.objects.CollateralAccessBean;
import com.ibm.commerce.marketingcenter.events.dbobjects.CpgnLog;
import com.ibm.commerce.marketingcenter.events.dbobjects.CpgnStats;
import com.ibm.commerce.marketingcenter.objects.EMarketingSpotAccessBean;
import com.ibm.commerce.membergroup.commands.CheckCurrentUserInMemberGroupCmd;
import com.ibm.commerce.order.objects.OrderAccessBean;
import com.ibm.commerce.order.objects.OrderItemAccessBean;
import com.ibm.commerce.price.utils.CurrencyManager;
import com.ibm.commerce.price.utils.MonetaryAmount;
import com.ibm.commerce.ras.ECTrace;
import com.ibm.commerce.ras.ECTraceIdentifiers;
import com.ibm.commerce.rule.Action;
import com.ibm.commerce.rules.PersonalizationRuleServerInfo;
import com.ibm.commerce.rules.RuleServiceKey;
import com.ibm.commerce.rules.RulesSystemDataModel;
import com.ibm.commerce.rules.commands.CreatePersonalizationRuleServiceCommand;
import com.ibm.commerce.rules.commands.InvokePersonalizationRuleServiceCommand;
import com.ibm.commerce.rules.helpers.CatalogGroupResultBean;
import com.ibm.commerce.rules.likeminds.LMRecommender;
import com.ibm.commerce.rules.likeminds.WCSRecommendationServer;
import com.ibm.commerce.rules.objects.MerchantCenterInfrastructureJDBCHelperAccessBean;
import com.ibm.commerce.search.rulequery.SelectQuery;
import com.ibm.commerce.server.ConfigProperties;
import com.ibm.commerce.server.WcsApp;
import com.ibm.commerce.user.objects.MemberGroupAccessBean;
import java.io.File;
import java.math.BigDecimal;
import java.sql.Timestamp;
import java.util.Calendar;
import java.util.Date;
import java.util.Enumeration;
import java.util.Vector;

/**
 * The CampaignInitiativeEvaluateCmdImpl class implements the CampaignInitiativeEvaluateCmd interface to
 * provide the task command that evaluates a campaign initiative.
 * Different implementations of this interface
 * can be provided for different stores.
 */
public class CampaignInitiativeEvaluateCmdImpl extends TaskCommandImpl implements CampaignInitiativeEvaluateCmd, CampaignConstants, ConditionConstants {
  /**
   * Copyright.
   */
  public static final String COPYRIGHT = IBMCopyright.SHORT_COPYRIGHT;

  /**
   * e-Marketing spot name.
   */
  protected String _eMarketingSpotName = null;

  /**
   * e-Marketing spot ID.
   */
  protected Integer _eMarketingSpotId = null;

  /**
   * Maximum number of catalog entries in this e-marketing spot.
   */
  protected int _maximumNumberOfCatalogEntries = 20;

  /**
   * Maximum number of ad copies in this e-marketing spot.
   */
  protected int _maximumNumberOfCollateral = 20;

  /**
   * Maximum number of categories in this e-marketing spot.
   */
  protected int _maximumNumberOfCategories = 20;

  /**
   * An instance of the CheckCurrentUserInMemberGroupCmd object.
   */
  protected CheckCurrentUserInMemberGroupCmd _checkCurrentUserInMemberGroupCmd = null;

  /**
   * Campaign initiative WHEN condition a day of the week
   */
  protected String _dayOfWeek = null;

  /**
   * An instance of the OrderAccessBean object, used to check the shopping cart.
   */
  protected OrderAccessBean _shoppingCartOrderAccessBean = null;

  /**
   * An array of instances of the OrderItemAccessBean object, used to check the shopping cart.
   */
  protected OrderItemAccessBean[] _shoppingCartOrderItems = null;

  /**
   * An array of instances of the CatalogGroupResultBean object, used to check the shopping cart.
   */
  protected CatalogGroupResultBean[][] _shoppingCartCatalogGroups = null;

  /**
   * The language of the catalog group in the shopping cart.
   */
  protected String _shoppingCartCatalogGroupsLanguage = null;

  /**
   * An array of instances of the OrderItemAccessBean object, used to check the purchase history.
   */
  protected OrderItemAccessBean[] _purchaseHistoryOrderItems = null;

  /**
   * An array of instances of the CatalogGroupResultBean object, used to check the purchase history.
   */
  protected CatalogGroupResultBean[][] _purchaseHistoryCatalogGroups = null;

  /**
   * The language of the catalog group in the purchase history.
   */
  protected String _purchaseHistoryCatalogGroupsLanguage = null;

  /**
   * An array of the categories in the purchase history.
   */
  protected String[] _purchaseHistoryCategories = null;

  /**
   * An instance of the CampaignInitiativeContext object.
   */
  protected CampaignInitiativeContext _campaignInitiativeContext = null;

  /**
   * The current total amount in the shopping cart.
   */
  protected BigDecimal _shoppingCartTotal = null;

  /**
   * An instance of the SelectQuery object.
   */
  protected SelectQuery _selectQuery = null;

  /**
   * Catalog entry IDs used in the evaluation of this initiative on what to show in the e-marketing spot.
   */
  protected Vector _catalogEntryIds = null;

  /**
   * Collateral names used in the evaluation of this initiative on what to show in the e-marketing spot.
   */
  protected Vector _collateralNames = null;

  /**
   * Category names used in the evaluation of this initiative on what to show in the e-marketing spot.
   */
  protected Vector _categoryNames = null;

  /**
   * Catalog entry results used in the evaluation of this initiative on what to show in the e-marketing spot.
   */
  protected Vector _catalogEntryResults = null;

  /**
   * Collateral results used in the evaluation of this initiative on what to show in the e-marketing spot.
   */
  protected Vector _collateralResults = null;

  /**
   * Category results used in the evaluation of this initiative on what to show in the e-marketing spot.
   */
  protected Vector _categoryResults = null;

  /**
   * Campaign initiative log results.
   */
  protected String _campaignLogResults = null;

  /**
   * Campaign initiative log segments.
   */
  protected String _campaignLogSegments = null;

  /**
   * An instance of the InvokePersonalizationRuleServiceCommand object.
   */
  protected InvokePersonalizationRuleServiceCommand _invokePersonalizationRuleServiceCommand = null;

  /**
   * The addCatalogEntry method is used to add catalog entries to the e-marketing spot,
   * after all conditions are being successfully evaluated.
   *
   * @param catalogEntryId  The ID of the catalog entry.
   * @param initiativeId  The ID of the campaign initiative.
   */
  protected void addCatalogEntry (String catalogEntryId, Integer initiativeId) {
	boolean found = false;
	for (int i=0; i<_catalogEntryResults.size(); i++) {
		EMarketingSpot.CatalogEntry catalogEntry = (EMarketingSpot.CatalogEntry) _catalogEntryResults.elementAt(i);
		if (catalogEntryId.equals(catalogEntry.getCatalogEntryID())) {
			found = true;
		}
	}
	if (!found) {
		EMarketingSpot.CatalogEntry catalogEntry = new EMarketingSpot.CatalogEntry();
		catalogEntry.setCatalogEntryID(catalogEntryId);
		//catalogEntry.setPartNumber(catalogEntryId);
		catalogEntry.setCommandContext(getCommandContext());
		catalogEntry.setInitiativeId(initiativeId);
		try {
			catalogEntry.populate();
			_catalogEntryResults.addElement(catalogEntry);
		}
		catch (Exception e) {
		}
		appendCampaignLogResults(catalogEntryId);
	}
  }

  /**
   * The addCategory method is used to add categories to the e-marketing spot,
   * after all conditions are being successfully evaluated.
   *
   * @param categoryName  The name of the category.
   * @param initiativeId  The ID of the campaign initiative.
   * @param memberIds  A vector of member IDs associated with the catalog used in the current store.
   */
  protected void addCategory (String categoryName, Integer initiativeId, Vector memberIds) {
	for (int j=0; j<memberIds.size(); j++) {
		Long memberId = (Long) memberIds.elementAt(j);
		try {
			CatalogGroupAccessBean catalogGroup = new CatalogGroupAccessBean().findByMemberIdAndIdentifier(memberId, categoryName);
			if (catalogGroup == null) {
				throw new ECApplicationException();
			}

			EMarketingSpot.Category category = new EMarketingSpot.Category(catalogGroup, getCommandContext());
			category.setInitiativeId(initiativeId);
			category.populate();
			_categoryResults.addElement(category);
			appendCampaignLogResults(catalogGroup.getCatalogGroupReferenceNumber());
		}
		catch (Exception e) {
		}
	}
  }

  /**
   * The addCollateral method is used to add ad copies to the e-marketing spot,
   * after all conditions are being successfully evaluated.
   *
   * @param collateralName  The name of the ad copies.
   * @param initiativeId  The ID of the campaign initiative.
   * @param storeId  The ID of the current store.
   * @param languageId  The ID of the language that is currently used.
   */
  protected void addCollateral (String collateralName, Integer initiativeId, Integer storeId, Integer languageId) {
	boolean found = false;
	for (int i=0; i<_collateralResults.size(); i++) {
		EMarketingSpot.Collateral collateral = (EMarketingSpot.Collateral) _collateralResults.elementAt(i);
		if (collateralName.equals(collateral.getName())) {
			found = true;
		}
	}
	if (!found) {
		try {
			CollateralAccessBean collateralAccessBean = new CollateralAccessBean().findByStoreAndName(storeId, collateralName);
			if (collateralAccessBean != null) {
				EMarketingSpot.Collateral collateral = new EMarketingSpot.Collateral();
				collateral.setCollateralID(collateralAccessBean.getCollateralIdInEJBType());
				collateral.setLanguageId(languageId);
				collateral.setInitiativeId(initiativeId);
				collateral.populate();
				_collateralResults.addElement(collateral);
				appendCampaignLogResults(collateralAccessBean.getCollateralId());
			}
		}
		catch (Exception e) {
		}
	}
  }

  /**
   * This evaluate method is used to evaluate open conditions using rule service.
   * A flag is returned indicates whether the evaluation of all the conditions
   * is successful or not.
   *
   * @param name  The name of the condition.
   * @param parameters  The parameters of the condition.
   *
   * @return <code>true</code> if the evaluation is successful.
   */
  public boolean evaluate (String name, OpenCondition.Parameter[] parameters) {
	CampaignInitiativeContext context = getCampaignInitiativeContext();
	context.setResult(false);
	context.setConditionName(name);
	context.setParameters(parameters);
	return invokeService(SERVICE_CAMPAIGN_INITIATIVE_EVALUATOR, context);
  }

  /**
   * This evaluate method is used to evaluate simple conditions.
   * A flag is returned indicates whether the evaluation of all the conditions
   * is successful or not.
   *
   * @param variable  The condition variable, indicates the type of WHICH condition to be evaluated.
   * @param operator  The operator of the condition.
   * @param value  The value of the condition.
   * @param qualifiers  The qualifiers of the condition.
   *
   * @return <code>true</code> if the evaluation is successful.
   */
  public boolean evaluate (String variable, String operator, String value, SimpleCondition.Qualifier[] qualifiers) {
	boolean result = false;
	if (VARIABLE_SEGMENT.equals(variable)) {
		result = evaluateSegmentCondition(operator, value, qualifiers);
	}
	else if (VARIABLE_SHOPPING_CART_SKU.equals(variable)) {
		result = evaluateShoppingCartSku(operator, value, qualifiers);
	}
	else if (VARIABLE_SHOPPING_CART_CATEGORY.equals(variable)) {
		result = evaluateShoppingCartCategory(operator, value, qualifiers);
	}
	else if (VARIABLE_PURCHASE_HISTORY_SKU.equals(variable)) {
		result = evaluatePurchaseHistorySku(operator, value, qualifiers);
	}
	else if (VARIABLE_PURCHASE_HISTORY_CATEGORY.equals(variable)) {
		result = evaluatePurchaseHistoryCategory(operator, value, qualifiers);
	}
	else if (VARIABLE_SHOPPING_CART_TOTAL.equals(variable)) {
		result = evaluateShoppingCartTotal(operator, value, qualifiers);
	}
	else if (VARIABLE_DAY_OF_WEEK.equals(variable)) {
		result = evaluateDayOfWeek(operator, value, qualifiers);
	}
	return result;
  }

  /**
   * The evaluateDayOfWeek method is used to evaluate the WHEN condition.
   * A flag is returned indicates whether the evaluation of all the conditions
   * is successful or not.
   *
   * @param operator  The operator of the condition.
   * @param value  The value of the condition.
   * @param qualifiers  The qualifiers of the condition.
   *
   * @return <code>true</code> if the evaluation is successful.
   */
  protected boolean evaluateDayOfWeek (String operator, String value, SimpleCondition.Qualifier[] qualifiers) {
	return ConditionUtil.evaluateString(getDayOfWeek(), operator, value);
  }

  /**
   * The evaluateDiscount method is used to evaluate the WHAT condition of discount advertisement type.
   * A flag is returned indicates whether the evaluation of all the conditions
   * is successful or not.
   *
   * @param discountUsage  The details of the discount.
   * @param discountCode  The code of the discount.
   *
   * @return <code>true</code> if the evaluation is successful.
   */
  protected boolean evaluateDiscount (String discountUsage, String discountCode) throws Exception {
	if (discountUsage.equals("") || discountCode.equals("")) {
		// unable to find discount
		return false;
	}

	// get discount info
	CalculationCodeAccessBean calculationCodeAccessBean = new CalculationCodeAccessBean().findByCalculationUsageCodeAndStoreEntity(new Integer(discountUsage), discountCode, getCommandContext().getStoreId());
	if (calculationCodeAccessBean == null) {
		// unable to find calculation code
		return false;
	}
	CalculationRuleAccessBean calculationRuleAccessBean;
	Enumeration ruleEnum = new CalculationRuleAccessBean().findByCalculationCode(calculationCodeAccessBean.getCalculationCodeIdInEJBType());
	if (ruleEnum.hasMoreElements()) {
		calculationRuleAccessBean = (CalculationRuleAccessBean)ruleEnum.nextElement();
	}
	else {
		// unable to find calculation rule
		return false;
	}

	// check start and end date
	Timestamp startDate = calculationCodeAccessBean.getStartDateInEJBType();
	Timestamp endDate = calculationCodeAccessBean.getEndDateInEJBType();
	Timestamp currentTime = new Timestamp(new Date().getTime());
	if (startDate == null) startDate = calculationRuleAccessBean.getStartDateInEJBType();
	if (endDate == null) endDate = calculationRuleAccessBean.getEndDateInEJBType();
	if (startDate == null) {
		return false;
	}
	if (endDate == null) {
		if (currentTime.before(startDate)) {
			return false;
		}
	}
	else {
		if (currentTime.before(startDate) || currentTime.after(endDate)) {
			return false;
		}
	}

	// check status
	final Integer activeDiscount = new Integer(1);
	Integer discountStatus = calculationCodeAccessBean.getPublishedInEJBType();
	if (discountStatus.compareTo(activeDiscount) != 0) {
		return false;
	}

	// check target group
	String targetGroup = null;
	boolean result = false;
	Enumeration ruleGroupEnum = new CalculationRuleMemberGroupRelationshipAccessBean().findByCalculationRule(calculationRuleAccessBean.getCalculationRuleIdInEJBType());
	if (ruleGroupEnum.hasMoreElements()) {
		while (ruleGroupEnum.hasMoreElements()) {
			CalculationRuleMemberGroupRelationshipAccessBean calculationRuleMemberGroupRelationshipAccessBean = (CalculationRuleMemberGroupRelationshipAccessBean)ruleGroupEnum.nextElement();
			if (calculationRuleMemberGroupRelationshipAccessBean != null) {
				MemberGroupAccessBean mgab = new MemberGroupAccessBean();
				mgab.setInitKey_MemberId(calculationRuleMemberGroupRelationshipAccessBean.getMemberGroupId());
				targetGroup = mgab.getMbrGrpName();
				// check current member group in the enumeration with the current member group of the store (runtime)
				CheckCurrentUserInMemberGroupCmd checkCurrentUserInMemberGroupCmd = getCheckCurrentUserInMemberGroupCmd();
				checkCurrentUserInMemberGroupCmd.setMemberGroupName(targetGroup);
				try {
					checkCurrentUserInMemberGroupCmd.execute();
					if (checkCurrentUserInMemberGroupCmd.getResult()) {
						result = true;
						break;
					}
				}
				catch (Exception e) {
				}
			}
		}
	}
	else {
		result = true;
	}
	if (!result) {
		return false;
	}

	return true;
  }

  /**
   * The evaluateCoupon method is used to evaluate the WHAT condition of coupon advertisement type.
   * A flag is returned indicates whether the evaluation of all the conditions
   * is successful or not.
   *
   * @param collateralName  The name of the coupon promotion.
   *
   * @return <code>true</code> if the evaluation is successful.
   */
  protected boolean evaluateCoupon (String collateralName) throws Exception {
	if (collateralName.equals("")) {
		// unable to find collateral
		return false;
	}

	CollateralAccessBean collateralAccessBean = new CollateralAccessBean().findByStoreAndName(getCommandContext().getStoreId(), collateralName);
	if (collateralAccessBean == null) {
		// unable to find collateral
		return false;
	}

	// get url link from bean
	String collateralUrlLink = collateralAccessBean.getUrlLink();
	// parse coupon promotion name from url
	String couponPromoName = "";
	final String promoParameter = "?promoName=";
	final String storeIdParameter = "&storeId=";
	int paramIndex = collateralUrlLink.indexOf(promoParameter);
	if (paramIndex > -1) {
		couponPromoName = collateralUrlLink.substring(paramIndex + promoParameter.length(), collateralUrlLink.length());
		paramIndex = couponPromoName.indexOf(storeIdParameter);
		if (paramIndex > -1) {
			couponPromoName = couponPromoName.substring(0, paramIndex);
		}
	}

	// get coupon promotion info
	CouponPromotionAccessBean couponPromotionAccessBean = new CouponPromotionAccessBean().findByPromoNameStoreId(couponPromoName, getCommandContext().getStoreId());
	if (couponPromotionAccessBean == null) {
		// unable to find coupon promotion
		return false;
	}

	// check start/end dates
	Timestamp startDate = couponPromotionAccessBean.getStartDate();
	Timestamp endDate = couponPromotionAccessBean.getEndDate();
	Timestamp currentTime = new Timestamp(new Date().getTime());
	if (startDate == null) {
		return false;
	}
	if (endDate == null) {
		if (currentTime.before(startDate)) {
			return false;
		}
	}
	else {
		if (currentTime.before(startDate) || currentTime.after(endDate)) {
			return false;
		}
	}

	// check status
	final String activeCoupon = "A";
	String couponStatus = couponPromotionAccessBean.getStatus();
	if (!couponStatus.equals(activeCoupon)) {
		return false;
	}

	// check number of coupons offered
	Integer totalOffer = couponPromotionAccessBean.getTotalNumOffer();
	Integer numberOffered = couponPromotionAccessBean.getNumOffered();
	if ((totalOffer.compareTo(new Integer(-1)) != 0) && (numberOffered.compareTo(totalOffer) >= 0)) {
		return false;
	}

	return true;
  }

  /**
   * The evaluatePurchaseHistoryCategory method is used to evaluate the WHICH condition,
   * checking the purchase history on the specified category.
   * A flag is returned indicates whether the evaluation of all the conditions
   * is successful or not.
   *
   * @param operator  The operator of the condition.
   * @param value  The operator of the condition.
   * @param qualifiers  The qualifiers of the condition.
   *
   * @return <code>true</code> if the evaluation is successful.
   */
  protected boolean evaluatePurchaseHistoryCategory (String operator, String value, SimpleCondition.Qualifier[] qualifiers) {
	String language = null;
	if (qualifiers != null) {
		for (int i=0; i<qualifiers.length; i++) {
			SimpleCondition.Qualifier qualifier = qualifiers[i];
			if (qualifier != null) {
				if (QUALIFIER_LANGUAGE.equals(qualifier.getName())) {
					language = qualifier.getData();
				}
			}
		}
	}
	CatalogGroupResultBean[][] purchaseHistoryCatalogGroups = getPurchaseHistoryCatalogGroups(language);
	boolean result = false;
	if (purchaseHistoryCatalogGroups != null) {
		for (int i=0; i<purchaseHistoryCatalogGroups.length; i++) {
			CatalogGroupResultBean[] groups = purchaseHistoryCatalogGroups[i];
			if (groups != null) {
				for (int j=0; j<groups.length; j++) {
					if (groups[j].getName().equals(value)) {
						result = true;
					}
				}
			}
		}
	}
	if (OPERATOR_NOT_EQUAL_TO.equals(operator)) {
		result = !result;
	}
	return result;
  }

  /**
   * The evaluatePurchaseHistorySku method is used to evaluate the WHICH condition,
   * checking the purchase history on the specified SKU.
   * A flag is returned indicates whether the evaluation of all the conditions
   * is successful or not.
   *
   * @param operator  The operator of the condition.
   * @param value  The operator of the condition.
   * @param qualifiers  The qualifiers of the condition.
   *
   * @return <code>true</code> if the evaluation is successful.
   */
  protected boolean evaluatePurchaseHistorySku (String operator, String value, SimpleCondition.Qualifier[] qualifiers) {
	OrderItemAccessBean orderItemAccessBeans[] = getPurchaseHistoryOrderItems();
	boolean result = false;
	for (int i=0; i<orderItemAccessBeans.length; i++) {
		String sku = null;
		try {
			sku = orderItemAccessBeans[i].getPartNumber();
		}
		catch (Exception e) {
		}
		if (sku != null && sku.equalsIgnoreCase(value)) {
			result = true;
			break;
		}
	}
	if (OPERATOR_NOT_EQUAL_TO.equals(operator)) {
		result = !result;
	}
	return result;
  }

  /**
   * The evaluateSegmentCondition method is used to evaluate the WHO condition,
   * comparing the target customer with the current customer in store.
   * A flag is returned indicates whether the evaluation of all the conditions
   * is successful or not.
   *
   * @param operator  The operator of the condition.
   * @param value  The operator of the condition.
   * @param qualifiers  The qualifiers of the condition.
   *
   * @return <code>true</code> if the evaluation is successful.
   */
  protected boolean evaluateSegmentCondition (String operator, String value, SimpleCondition.Qualifier[] qualifiers) {
	boolean result = false;
	CheckCurrentUserInMemberGroupCmd checkCurrentUserInMemberGroupCmd = getCheckCurrentUserInMemberGroupCmd();
	checkCurrentUserInMemberGroupCmd.setMemberGroupName(value);
	try {
		checkCurrentUserInMemberGroupCmd.execute();
		result = checkCurrentUserInMemberGroupCmd.getResult();
		if (result)
		 {
		  if (_campaignLogSegments == null)
		   {
			_campaignLogSegments = "";
		   }
		  else
		   {
			_campaignLogSegments += ",";
		   }
		  _campaignLogSegments += value;
		 }

	}
	catch (Exception e) {
	}
	if (OPERATOR_NOT_EQUAL_TO.equals(operator)) {
		result = !result;
	}
	return result;
  }

  /**
   * The evaluateShoppingCartCategory method is used to evaluate the WHICH condition,
   * checking the shopping cart if it contains the specified category.
   * A flag is returned indicates whether the evaluation of all the conditions
   * is successful or not.
   *
   * @param operator  The operator of the condition.
   * @param value  The operator of the condition.
   * @param qualifiers  The qualifiers of the condition.
   *
   * @return <code>true</code> if the evaluation is successful.
   */
  protected boolean evaluateShoppingCartCategory (String operator, String value, SimpleCondition.Qualifier[] qualifiers) {
	String language = null;
	if (qualifiers != null) {
		for (int i=0; i<qualifiers.length; i++) {
			SimpleCondition.Qualifier qualifier = qualifiers[i];
			if (qualifier != null) {
				if (QUALIFIER_LANGUAGE.equals(qualifier.getName())) {
					language = qualifier.getData();
				}
			}
		}
	}
	CatalogGroupResultBean[][] shoppingCartCatalogGroups = getShoppingCartCatalogGroups(language);
	boolean result = false;
	if (shoppingCartCatalogGroups != null) {
		for (int i=0; i<shoppingCartCatalogGroups.length; i++) {
			CatalogGroupResultBean[] groups = shoppingCartCatalogGroups[i];
			if (groups != null) {
				for (int j=0; j<groups.length; j++) {
					if (groups[j].getName().equals(value)) {
						result = true;
					}
				}
			}
		}
	}
	if (OPERATOR_NOT_EQUAL_TO.equals(operator)) {
		result = !result;
	}
	return result;
  }

  /**
   * The evaluateShoppingCartSku method is used to evaluate the WHICH condition,
   * checking the shopping cart if it contains the specified SKU.
   * A flag is returned indicates whether the evaluation of all the conditions
   * is successful or not.
   *
   * @param operator  The operator of the condition.
   * @param value  The operator of the condition.
   * @param qualifiers  The qualifiers of the condition.
   *
   * @return <code>true</code> if the evaluation is successful.
   */
  protected boolean evaluateShoppingCartSku (String operator, String value, SimpleCondition.Qualifier[] qualifiers) {
	OrderItemAccessBean orderItemAccessBeans[] = getShoppingCartOrderItems();
	boolean result = false;
	for (int i=0; i<orderItemAccessBeans.length; i++) {
		String sku = null;
		try {
			sku = orderItemAccessBeans[i].getPartNumber();
		}
		catch (Exception e) {
		}
		if (sku != null && sku.equalsIgnoreCase(value)) {
			result = true;
			break;
		}
	}
	if (OPERATOR_NOT_EQUAL_TO.equals(operator)) {
		result = !result;
	}
	return result;
  }

  /**
   * The evaluateShoppingCartTotal method is used to evaluate the WHICH condition,
   * comparing the shopping cart total amount with the specified amount in the condition.
   * A flag is returned indicates whether the evaluation of all the conditions
   * is successful or not.
   *
   * @param operator  The operator of the condition.
   * @param value  The operator of the condition.
   * @param qualifiers  The qualifiers of the condition.
   *
   * @return <code>true</code> if the evaluation is successful.
   */
  protected boolean evaluateShoppingCartTotal (String operator, String value, SimpleCondition.Qualifier[] qualifiers) {
	String currency = null;
	if (qualifiers != null) {
		for (int i=0; i<qualifiers.length; i++) {
			SimpleCondition.Qualifier qualifier = qualifiers[i];
			if (qualifier != null) {
				if (QUALIFIER_CURRENCY.equals(qualifier.getName())) {
					currency = qualifier.getData();
				}
			}
		}
	}
	return ConditionUtil.evaluateBigDecimal(getShoppingCartTotal(currency), operator, CampaignUtil.toBigDecimal(value));
  }

  /**
   * Gets the campaign initiative context object.
   *
   * @return The campaign initiative context object.
   */
  protected CampaignInitiativeContext getCampaignInitiativeContext () {
	if (_campaignInitiativeContext == null) {
		_campaignInitiativeContext = new CampaignInitiativeContext();
		_campaignInitiativeContext.setCommandContext(getCommandContext());
	}
	return _campaignInitiativeContext;
  }

  /**
   * Gets the catalog entries to be displayed in the e-marketing spot.
   *
   * @return The catalog entries to be displayed in the e-marketing spot.
   */
  public EMarketingSpot.CatalogEntry[] getCatalogEntries () {
	if (_catalogEntryResults != null) {
		EMarketingSpot.CatalogEntry[] catalogEntries = new EMarketingSpot.CatalogEntry[_catalogEntryResults.size()];
		_catalogEntryResults.copyInto(catalogEntries);
		return catalogEntries;
	}
	return null;
  }

  /**
   * Gets the categories to be displayed in the e-marketing spot.
   *
   * @return The categories to be displayed in the e-marketing spot.
   */
  public EMarketingSpot.Category[] getCategories () {
	if (_categoryResults != null) {
		EMarketingSpot.Category[] categories = new EMarketingSpot.Category[_categoryResults.size()];
		_categoryResults.copyInto(categories);
		return categories;
	}
	return null;
  }

  /**
   * Gets the category name given the category identifier.
   *
   * @param categoryIdentifier  The category identifier.
   *
   * @return The name of the category.
   */
  protected String getCategoryName (String categoryIdentifier) {
	String categoryName = "";
	try {
		Vector memberIds = new Vector();
		Enumeration list = new CatalogAccessBean().findByStoreId(getCommandContext().getStoreId());
		while (list.hasMoreElements()) {
			CatalogAccessBean catalogAccessBean = (CatalogAccessBean) list.nextElement();
			memberIds.addElement(catalogAccessBean.getMemberIdInEJBType());
		}
		for (int i=0; i<memberIds.size(); i++) {
			Long memberId = (Long) memberIds.elementAt(i);
			CatalogGroupAccessBean catalogGroup = new CatalogGroupAccessBean().findByMemberIdAndIdentifier(memberId, categoryIdentifier);
			if (catalogGroup != null) {
				categoryName = catalogGroup.getDescription(getCommandContext().getLanguageId()).getName();
				break;
			}
		}
	}
	catch (Exception e) {
	}
	return categoryName;
  }

  /**
   * Gets the current user in the member group.
   *
   * @return The CheckCurrentUserInMemberGroupCmd object of the current condition.
   */
  protected CheckCurrentUserInMemberGroupCmd getCheckCurrentUserInMemberGroupCmd () {
	if (_checkCurrentUserInMemberGroupCmd == null) {
		try {
			_checkCurrentUserInMemberGroupCmd = (CheckCurrentUserInMemberGroupCmd) CommandFactory.createCommand(CheckCurrentUserInMemberGroupCmd.Name, getStoreId());
			_checkCurrentUserInMemberGroupCmd.setCommandContext(getCommandContext());
		}
		catch (Exception e) {
		}
	}
	_checkCurrentUserInMemberGroupCmd.setMemberGroupCondition(null);
	return _checkCurrentUserInMemberGroupCmd;
  }

  /**
   * Invokes the personalization rule service.
   *
   * @return The InvokePersonalizationRuleServiceCommand object.
   */
  protected InvokePersonalizationRuleServiceCommand getInvokePersonalizationRuleServiceCommand() {
	if (_invokePersonalizationRuleServiceCommand == null) {
		try {
			_invokePersonalizationRuleServiceCommand = (InvokePersonalizationRuleServiceCommand)  CommandFactory.createCommand(InvokePersonalizationRuleServiceCommand.class.getName(), getStoreId());
			_invokePersonalizationRuleServiceCommand.setCommandContext(getCommandContext());
		}
		catch (Exception e) {
		}
	}
	return _invokePersonalizationRuleServiceCommand;
  }

  /**
   * Gets the ad copies to be displayed in the e-marketing spot.
   *
   * @return The ad copies to be displayed in the e-marketing spot.
   */
  public EMarketingSpot.Collateral[] getCollateral () {
	if (_collateralResults != null) {
		EMarketingSpot.Collateral[] collateral = new EMarketingSpot.Collateral[_collateralResults.size()];
		_collateralResults.copyInto(collateral);
		return collateral;
	}
	return null;
  }

  /**
   * Gets the current day of the week from the java Calendar object.
   *
   * @return The current day of the week constant.
   */
  protected String getDayOfWeek () {
	if (_dayOfWeek == null) {
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(new Date());
		switch (calendar.get(Calendar.DAY_OF_WEEK)) {
			case Calendar.SUNDAY: {
				_dayOfWeek = SUNDAY;
				break;
			}
			case Calendar.MONDAY: {
				_dayOfWeek = MONDAY;
				break;
			}
			case Calendar.TUESDAY: {
				_dayOfWeek = TUESDAY;
				break;
			}
			case Calendar.WEDNESDAY: {
				_dayOfWeek = WEDNESDAY;
				break;
			}
			case Calendar.THURSDAY: {
				_dayOfWeek = THURSDAY;
				break;
			}
			case Calendar.FRIDAY: {
				_dayOfWeek = FRIDAY;
				break;
			}
			case Calendar.SATURDAY: {
				_dayOfWeek = SATURDAY;
				break;
			}
		}
	}
	return _dayOfWeek;
  }

  /**
   * Gets the e-marketing spot ID.
   *
   * @return The ID of the e-marketing spot.
   */
  public Integer getEMarketingSpotId () {
	return _eMarketingSpotId;
  }

  /**
   * Gets the catalog groups from the purchase history of the store.
   *
   * @param language  Language description of the catalog group.
   *
   * @return An array of resulted catalog groups.
   */
  protected CatalogGroupResultBean[][] getPurchaseHistoryCatalogGroups (String language) {
	if (_purchaseHistoryCatalogGroups == null || _purchaseHistoryCatalogGroupsLanguage == null || !_purchaseHistoryCatalogGroupsLanguage.equals(language)) {
		OrderItemAccessBean[] purchaseHistoryOrderItems = getPurchaseHistoryOrderItems();
		_purchaseHistoryCatalogGroups = new CatalogGroupResultBean[purchaseHistoryOrderItems.length * 2][];
		Integer languageId = CampaignUtil.getLanguageIdFromLocaleName(language);
		MerchantCenterInfrastructureJDBCHelperAccessBean helperAccessBean = new MerchantCenterInfrastructureJDBCHelperAccessBean();
		for (int i=0; i<purchaseHistoryOrderItems.length; i++) {
			try {
				Long catalogEntryId = purchaseHistoryOrderItems[i].getCatalogEntryIdInEJBType();
				_purchaseHistoryCatalogGroups[i * 2] = helperAccessBean.findCatGroupsByCatEntryId(catalogEntryId, new Long(languageId.longValue()), false);
				_purchaseHistoryCatalogGroups[(i * 2) + 1] = new CatalogGroupResultBean[0];
				CatalogEntryAccessBean catalogEntry = new CatalogEntryAccessBean();
				Enumeration parents = catalogEntry.findByChildCatalogEntry(catalogEntryId);
				if (parents.hasMoreElements()) {
					CatalogEntryAccessBean parentCatalogEntry = (CatalogEntryAccessBean) parents.nextElement();
					Long parentId = parentCatalogEntry.getCatalogEntryReferenceNumberInEJBType();
					_purchaseHistoryCatalogGroups[(i * 2) + 1] = helperAccessBean.findCatGroupsByCatEntryId(parentId, new Long(languageId.longValue()), false);
				}
			}
			catch (Exception e) {
			}
		}
		_purchaseHistoryCatalogGroupsLanguage = language;
	}
	return _purchaseHistoryCatalogGroups;
  }

  /**
   * Gets the categories from the purchase history of the store.
   *
   * @param language  Language description of the category.
   *
   * @return An array of resulted categories.
   */
  protected String[] getPurchaseHistoryCategories (String language) {
	if (_purchaseHistoryCategories == null) {
		CatalogGroupResultBean[][] purchaseHistoryCatalogGroups = getPurchaseHistoryCatalogGroups(language);
		Vector purchaseHistoryCategoryVector = new Vector();
		if (purchaseHistoryCatalogGroups != null) {
			for (int i=0; i<purchaseHistoryCatalogGroups.length; i++) {
				CatalogGroupResultBean[] groups = purchaseHistoryCatalogGroups[i];
				if (groups != null) {
					for (int j=0; j<groups.length; j++) {
						String category = groups[j].getName();
						boolean found = false;
						for (int k=0; k<purchaseHistoryCategoryVector.size(); k++) {
							if (purchaseHistoryCategoryVector.elementAt(k).equals(category)) {
								found = true;
								break;
							}
						}
						if (!found) {
							purchaseHistoryCategoryVector.addElement(category);
						}
					}
				}
			}
		}
		_purchaseHistoryCategories = new String[purchaseHistoryCategoryVector.size()];
		purchaseHistoryCategoryVector.copyInto(_purchaseHistoryCategories);
	}
	return _purchaseHistoryCategories;
  }

  /**
   * Gets the order items from the purchase history of the store.
   *
   * @return An array of resulted order items.
   */
  protected OrderItemAccessBean[] getPurchaseHistoryOrderItems () {
	if (_purchaseHistoryOrderItems == null) {
		Vector orderItemVector = new Vector();
		try {
			Enumeration list = new OrderAccessBean().findWithPushDownQuery("STATUS NOT IN ('P','X','I') AND MEMBER_ID=" + getUserId() + " AND STOREENT_ID=" + getStoreId());
			while (list.hasMoreElements()) {
				OrderAccessBean orderAccessBean = (OrderAccessBean) list.nextElement();
				Enumeration list2 = new OrderItemAccessBean().findByOrder(orderAccessBean.getOrderIdInEJBType());
				while (list2.hasMoreElements()) {
					orderItemVector.addElement(list2.nextElement());
				}
			}
		}
		catch (Exception e) {
		}
		_purchaseHistoryOrderItems = new OrderItemAccessBean[orderItemVector.size()];
		orderItemVector.copyInto(_purchaseHistoryOrderItems);
	}
	return _purchaseHistoryOrderItems;
  }

  /**
   * Gets the integer constant to be used in the SelectQuery class.
   *
   * @param operator  The operator of the condition.
   *
   * @return An integer constant to be used in the SelectQuery class.
   */
  protected int getSelectQueryOperator (String operator) {
	int queryOperator = SelectQuery.EQ_Operator;
	if (OPERATOR_EQUAL_TO.equals(operator)) {
		queryOperator = SelectQuery.EQ_Operator;
	}
	else if (OPERATOR_LESS_THAN.equals(operator)) {
		queryOperator = SelectQuery.LT_Operator;
	}
	else if (OPERATOR_GREATER_THAN.equals(operator)) {
		queryOperator = SelectQuery.GT_Operator;
	}
	return queryOperator;
  }

  /**
   * Gets the catalog groups from the shopping cart of the store.
   *
   * @param language  Language description of the catalog group.
   *
   * @return An array of resulted catalog groups.
   */
  protected CatalogGroupResultBean[][] getShoppingCartCatalogGroups (String language) {
	if (_shoppingCartCatalogGroups == null || _shoppingCartCatalogGroupsLanguage == null || !_shoppingCartCatalogGroupsLanguage.equals(language)) {
		OrderItemAccessBean[] shoppingCartOrderItems = getShoppingCartOrderItems();
		_shoppingCartCatalogGroups = new CatalogGroupResultBean[shoppingCartOrderItems.length * 2][];
		Integer languageId = CampaignUtil.getLanguageIdFromLocaleName(language);
		MerchantCenterInfrastructureJDBCHelperAccessBean helperAccessBean = new MerchantCenterInfrastructureJDBCHelperAccessBean();
		for (int i=0; i<shoppingCartOrderItems.length; i++) {
			try {
				Long catalogEntryId = shoppingCartOrderItems[i].getCatalogEntryIdInEJBType();
				_shoppingCartCatalogGroups[i * 2] = helperAccessBean.findCatGroupsByCatEntryId(catalogEntryId, new Long(languageId.longValue()), false);
				_shoppingCartCatalogGroups[(i * 2) + 1] = new CatalogGroupResultBean[0];
				CatalogEntryAccessBean catalogEntry = new CatalogEntryAccessBean();
				Enumeration parents = catalogEntry.findByChildCatalogEntry(catalogEntryId);
				if (parents.hasMoreElements()) {
					CatalogEntryAccessBean parentCatalogEntry = (CatalogEntryAccessBean) parents.nextElement();
					Long parentId = parentCatalogEntry.getCatalogEntryReferenceNumberInEJBType();
					_shoppingCartCatalogGroups[(i * 2) + 1] = helperAccessBean.findCatGroupsByCatEntryId(parentId, new Long(languageId.longValue()), false);
				}
			}
			catch (Exception e) {
			}
		}
		_shoppingCartCatalogGroupsLanguage = language;
	}
	return _shoppingCartCatalogGroups;
  }

  /**
   * Gets the order item from the shopping cart of the store.
   *
   * @return The resulted order item.
   */
  protected OrderAccessBean getShoppingCartOrderAccessBean () {
	if (_shoppingCartOrderAccessBean == null) {
		try {
			Enumeration list = new OrderAccessBean().findCurrentPendingOrdersByMemberAndStore(getUserId(), getStoreId());
			if (list == null || !list.hasMoreElements()) {
				list = new OrderAccessBean().findByStatusMemberAndStore("P", getUserId(), getStoreId());
			}
			Timestamp shoppingCartLastUpdate = null;
			while (list.hasMoreElements()) {
				OrderAccessBean orderAccessBean = (OrderAccessBean) list.nextElement();
				Timestamp lastUpdate = orderAccessBean.getLastUpdateInEJBType();
				if (_shoppingCartOrderAccessBean == null || lastUpdate.compareTo(shoppingCartLastUpdate) > 0) {
					_shoppingCartOrderAccessBean = orderAccessBean;
					shoppingCartLastUpdate = lastUpdate;
				}
			}
		}
		catch (Exception e) {
		}
	}
	if (_shoppingCartOrderAccessBean != null) {
		try {
		}
		catch (Exception e) {
		}
	}
	return _shoppingCartOrderAccessBean;
  }

  /**
   * Gets the order items from the shopping cart of the store.
   *
   * @return An array of the resulted order items.
   */
  protected OrderItemAccessBean[] getShoppingCartOrderItems () {
	if (_shoppingCartOrderItems == null) {
		OrderAccessBean shoppingCartOrderAccessBean = getShoppingCartOrderAccessBean();
		Vector orderItemVector = new Vector();
		try {
			if (shoppingCartOrderAccessBean != null) {
				Enumeration list = new OrderItemAccessBean().findByOrder(shoppingCartOrderAccessBean.getOrderIdInEJBType());
				while (list.hasMoreElements()) {
					orderItemVector.addElement(list.nextElement());
				}
			}
		}
		catch (Exception e) {
		}
		_shoppingCartOrderItems = new OrderItemAccessBean[orderItemVector.size()];
		orderItemVector.copyInto(_shoppingCartOrderItems);
	}
	return _shoppingCartOrderItems;
  }

  /**
   * Gets the total amount from the shopping cart of the store.
   *
   * @param currency  The currency used in the store.
   *
   * @return The total amount from the shopping cart of the store.
   */
  protected BigDecimal getShoppingCartTotal (String currency) {
	if (_shoppingCartTotal == null) {
		_shoppingCartTotal = new BigDecimal(0.0d);
		try {
			StoreAccessBean storeAccessBean = WcsApp.storeRegistry.find(getStoreId());
			Integer languageId = storeAccessBean.getLanguageIdInEJBType();
			CurrencyManager currencyManager = CurrencyManager.getInstance();
			if (currency == null) {
				currency = currencyManager.getDefaultCurrency(storeAccessBean, languageId);
			}
			OrderItemAccessBean[] orderItems = getShoppingCartOrderItems();
			if (orderItems != null) {
				for (int i=0; i<orderItems.length; i++) {
					OrderItemAccessBean orderItemAccessBean = orderItems[i];
					String orderItemCurrency = orderItemAccessBean.getCurrency();
					BigDecimal orderItemAmount = orderItemAccessBean.getTotalProductInEJBType();
					BigDecimal adjustment = orderItemAccessBean.getTotalAdjustmentInEJBType();
					if (adjustment != null) {
						orderItemAmount = orderItemAmount.add(adjustment);
					}
					if (orderItemCurrency.equals(currency)) {
						_shoppingCartTotal = _shoppingCartTotal.add(orderItemAmount);
					}
					else {
						MonetaryAmount convertedAmount = currencyManager.convert(new MonetaryAmount(orderItemAmount, orderItemCurrency), storeAccessBean, currency);
						_shoppingCartTotal = _shoppingCartTotal.add(convertedAmount.getValue());
					}
				}
			}
		}
		catch (Exception e) {
		}
	}
	return _shoppingCartTotal;
  }

  /**
   * The invokeService method is used to invoke the rule service to evaluate conditions.
   * A flag is returned indicates whether the rule service is started successfully or not.
   *
   * @param serviceName  The name of the rule service.
   * @param campaignInitiativeContext  An instance of the CampaignInitiativeContext object.
   *
   * @return <code>true</code> if the service has been successfully invoked.
   */
  protected boolean invokeService (String serviceName, CampaignInitiativeContext campaignInitiativeContext) {
	if (SERVICE_CAMPAIGN_INITIATIVE_EVALUATOR.equals(serviceName) && !RulesSystemDataModel.getInstance().ruleServiceConfigurationExists(PersonalizationRuleServerInfo.getName(), new RuleServiceKey(serviceName, getStoreId()))) {
		try {
			String directory = ConfigProperties.singleton().getValue("Instance/WCSInstallDir") + File.separator + "rules" + File.separator + "campaigns";
			directory = new File(directory).getCanonicalPath();
			directory += File.separator;
			CreatePersonalizationRuleServiceCommand createPersonalizationRuleServiceCommand = (CreatePersonalizationRuleServiceCommand) CommandFactory.createCommand(CreatePersonalizationRuleServiceCommand.class.getName(), getStoreId());
			createPersonalizationRuleServiceCommand.setCommandContext(getCommandContext());
			createPersonalizationRuleServiceCommand.setServiceName(serviceName);
			createPersonalizationRuleServiceCommand.setProjectPath(new File(directory + PROJECT_CAMPAIGN_INITIATIVE_EVALUATOR));
			createPersonalizationRuleServiceCommand.setAgents(new Integer(NdDeployRulesServiceConfig.DYNAMIC_NUM_AGENTS));
			createPersonalizationRuleServiceCommand.execute();
		}
		catch (Exception e) {
		}
	}
	try {
		InvokePersonalizationRuleServiceCommand invokePersonalizationRuleServiceCommand = getInvokePersonalizationRuleServiceCommand();
		invokePersonalizationRuleServiceCommand.setServiceName(serviceName);
		invokePersonalizationRuleServiceCommand.setInvocationContext(campaignInitiativeContext);
		invokePersonalizationRuleServiceCommand.execute();
	}
	catch (Exception e) {
	}
	return campaignInitiativeContext.getResult();
  }

  /**
   * Checks to see if the command is ready to call.  This method will
   * return <code>false</code> if the e-marketing spot name is null.
   *
   * @return <code>true</code> if the command is ready to go.
   */
  public boolean isReadyToCallExecute () {
	return super.isReadyToCallExecute() && _eMarketingSpotName != null;
  }

  /**
   * Performs the execution of the command.
   *
   * @param action  The type of action to be executed.
   */
  public void performAction (Action action) {
	String actionName = action.getName();
	if (ACTION_SUGGESTIVE_SELL.equals(actionName)) {
		performSuggestiveSell(action.getParameters());
	}
	else if (ACTION_AWARENESS_AD.equals(actionName)) {
		performAwarenessAd(action.getParameters());
	}
	else if (ACTION_DISCOUNT_AD.equals(actionName)) {
		performDiscountAd(action.getParameters());
	}
	else if (ACTION_COUPON_AD.equals(actionName)) {
		performCouponAd(action.getParameters());
	}
	else if (ACTION_CATEGORY_RECOMMENDATION.equals(actionName)) {
		performCategoryRecommendation(action.getParameters());
	}
	else if (ACTION_COLLABORATIVE_FILTERING.equals(actionName)) {
		performCollaborativeFiltering(action.getParameters());
	}
  }

  /**
   * Performs the execution of the command, used specific to general advertisements.
   *
   * @param parameters  The parameters of the action conditions.
   */
  protected void performAwarenessAd (Action.Parameter[] parameters) {
	if (parameters != null) {
		for (int i=0; i<parameters.length; i++) {
			if (ACTION_PARAMETER_COLLATERAL.equals(parameters[i].getName())) {
				_collateralNames.addElement(parameters[i].getValue());
			}
		}
	}
  }

  /**
   * Performs the execution of the command, used specific to discount advertisements.
   *
   * @param parameters  The parameters of the action conditions.
   */
  protected void performDiscountAd (Action.Parameter[] parameters) {
	String discountUsage = "";
	String discountCode = "";
	Vector uncheckNames = new Vector();
	if (parameters != null) {
		for (int i=0; i<parameters.length; i++) {
			if (ACTION_PARAMETER_DISCOUNT_USAGE.equals(parameters[i].getName())) {
				discountUsage = parameters[i].getValue();
			}
			if (ACTION_PARAMETER_DISCOUNT_CODE.equals(parameters[i].getName())) {
				discountCode = parameters[i].getValue();
			}
			if (ACTION_PARAMETER_DISCOUNT_COLLATERAL.equals(parameters[i].getName())) {
				uncheckNames.addElement(parameters[i].getValue());
			}
		}
	}
	// evaluate discount start date/end date and target customer group
	// if evaluation is successful, add collateral name to global vector
	try {
		if (evaluateDiscount(discountUsage, discountCode)) {
			_collateralNames = uncheckNames;
		}
	}
	catch (Exception e) {
	}
  }

  /**
   * Performs the execution of the command, used specific to coupon advertisements.
   *
   * @param parameters  The parameters of the action conditions.
   */
  protected void performCouponAd (Action.Parameter[] parameters) {
	if (parameters != null) {
		for (int i=0; i<parameters.length; i++) {
			if (ACTION_PARAMETER_COUPON_COLLATERAL.equals(parameters[i].getName())) {
				// evaluate coupon promotion start date/end date
				// if evaluation is successful, add collateral name to global vector
				try {
					if (evaluateCoupon(parameters[i].getValue())) {
						_collateralNames.addElement(parameters[i].getValue());
					}
				}
				catch (Exception e) {
				}
			}
		}
	}
  }

  /**
   * Performs the execution of the command, used specific to category recommendations.
   *
   * @param parameters  The parameters of the action conditions.
   */
  protected void performCategoryRecommendation (Action.Parameter[] parameters) {
	if (parameters != null) {
		for (int i=0; i<parameters.length; i++) {
			if (ACTION_PARAMETER_CATEGORY.equals(parameters[i].getName())) {
				_categoryNames.addElement(parameters[i].getValue());
			}
		}
	}
  }

  /**
   * Performs the execution of the command, used specific to collaborative filtering.
   *
   * @param parameters  The parameters of the action conditions.
   */
  protected void performCollaborativeFiltering (Action.Parameter[] parameters) {
	_catalogEntryIds = getWCSRecommendationServer().getRecommendations(getCommandContext(), _maximumNumberOfCatalogEntries, null, null);
  }

  /**
   * Performs the execution of the command.
   */
  public void performExecute () throws ECException {
	final String methodName = "performExecute";
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_MARKETING, getClass().getName(), methodName);
	try {
		Integer storeId = getStoreId();
		EMarketingSpotAccessBean eMarketingSpotAccessBean = new EMarketingSpotAccessBean().findByStoreIdAndName(storeId, _eMarketingSpotName);
		_catalogEntryResults = new Vector();
		_collateralResults = new Vector();
		_categoryResults = new Vector();

		if (eMarketingSpotAccessBean != null) {
			_eMarketingSpotId = eMarketingSpotAccessBean.getEMarketingSpotIdInEJBType();
			Vector cachedInitiatives = CampaignInitiativeCache.getCachedInitiatives(_eMarketingSpotId);
			for (int i=0; i<cachedInitiatives.size(); i++) {
				_selectQuery = new SelectQuery(getStoreId().intValue(), getCommandContext().getCurrentTradingAgreements(), getUserId());
				_selectQuery.setDistinct(true);
				_catalogEntryIds = new Vector();
				_collateralNames = new Vector();
				_categoryNames = new Vector();
				_campaignLogResults = null;
				_campaignLogSegments = null;

				CampaignInitiativeCache.CachedInitiative cachedInitiative = (CampaignInitiativeCache.CachedInitiative) cachedInitiatives.elementAt(i);
				if (cachedInitiative != null && cachedInitiative._rule != null) {
					cachedInitiative._rule.invoke(this, this);
					if (populateResults(cachedInitiative._initiativeId))
					 {
					  CpgnLog cpgnLog = new CpgnLog();
					  cpgnLog.addEvent(new Long(getCommandContext().getActivityId()),
						  cachedInitiative._campaignId,
						  cachedInitiative._campaignName,
						  cachedInitiative._initiativeId,
						  cachedInitiative._initiativeName,
						  null, // mpeTypeId,
						  cachedInitiative._resultType,
						  _eMarketingSpotId,
						  _eMarketingSpotName,
						  storeId,
						  _campaignLogSegments,
						  _campaignLogResults);
					  CpgnStats cpgnStats = new CpgnStats();
					  cpgnStats.addEvent(_eMarketingSpotId,
						  _eMarketingSpotName,
						  cachedInitiative._initiativeId,
						  cachedInitiative._initiativeName,
						  storeId,
						  null,  // mpeTypeId,
						  cachedInitiative._resultType);
					 }
				}
			}
		}
	}
	catch (Exception e) {
	}
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_MARKETING, getClass().getName(), methodName);
  }

  /**
   * Performs the execution of the command, used specific to suggestive sellings.
   *
   * @param parameters  The parameters of the action conditions.
   */
  protected void performSuggestiveSell (Action.Parameter[] parameters) {
	if (parameters != null) {
		for (int i=0; i<parameters.length; i++) {
			Action.Parameter parameter = parameters[i];
			String parameterName = parameter.getName();
			String parameterValue = parameter.getValue();
			try {
				if (ACTION_PARAMETER_QUERY_OPERATOR.equals(parameterName)) {
					if (ACTION_PARAMETER_VALUE_OR.equals(parameterValue)) {
						_selectQuery.addSelectOperator(SelectQuery.OR_Operator);
					}
					else if (ACTION_PARAMETER_VALUE_AND.equals(parameterValue)) {
						_selectQuery.addSelectOperator(SelectQuery.AND_Operator);
					}
				}
				else if (ACTION_PARAMETER_QUERY_TYPE.equals(parameterName)) {
					if (ACTION_PARAMETER_VALUE_SKU_QUERY.equals(parameterValue)) {
						_selectQuery.addRule(SelectQuery.CATENTRY_SKU_QueryType);
					}
					else if (ACTION_PARAMETER_VALUE_BOTH_QUERY.equals(parameterValue)) {
						_selectQuery.addRule(SelectQuery.BOTH_QueryType);
					}
					else if (ACTION_PARAMETER_VALUE_GENERIC_QUERY.equals(parameterValue)) {
						_selectQuery.addRule(SelectQuery.GENERIC_QueryType);
					}
				}
				else if (ACTION_PARAMETER_SKU.equals(parameterName)) {
					_selectQuery.addSelectAttribute(SelectQuery.CATENTRY_SKU_Attr, SelectQuery.EQ_Operator, parameterValue);
				}
				else if (ACTION_PARAMETER_SKU_PREFIX.equals(parameterName)) {
					_selectQuery.addSelectAttribute(SelectQuery.CATENTRY_SKU_Attr, SelectQuery.LEFT_LIKE_Operator, parameterValue, SelectQuery.UPPER_Function);
				}
				else if (ACTION_PARAMETER_CATEGORY.equals(parameterName)) {
					String catgrpdesc_name = getCategoryName(parameterValue);
					_selectQuery.addSelectAttribute(SelectQuery.CATGRPDESC_NAME_Attr, SelectQuery.EQ_Operator, catgrpdesc_name);
					Action.Parameter[] subParameters = parameter.getSubParameters();
					if (subParameters != null) {
						for (int j=0; j<subParameters.length; j++) {
							Action.Parameter subParameter = subParameters[j];
							if (ACTION_PARAMETER_LANGUAGE.equals(subParameter.getName())) {
								Integer languageId = CampaignUtil.getLanguageIdFromLocaleName(subParameter.getValue());
								if (languageId != null) {
									_selectQuery.addSelectAttribute(SelectQuery.CATGRPDESC_LANGUAGE_ID_Attr, SelectQuery.EQ_Operator, languageId.toString());
								}
							}
						}
					}
				}
				else if (ACTION_PARAMETER_PRODUCT_DESCRIPTION.equals(parameterName)) {
					_selectQuery.addSelectAttribute(SelectQuery.CATENTDESC_NAME_Attr, SelectQuery.LIKE_Operator, parameterValue, SelectQuery.UPPER_Function);
					Action.Parameter[] subParameters = parameter.getSubParameters();
					if (subParameters != null) {
						for (int j=0; j<subParameters.length; j++) {
							Action.Parameter subParameter = subParameters[j];
							if (ACTION_PARAMETER_LANGUAGE.equals(subParameter.getName())) {
								Integer languageId = CampaignUtil.getLanguageIdFromLocaleName(subParameter.getValue());
								if (languageId != null) {
									_selectQuery.addSelectAttribute(SelectQuery.CATENTDESC_LANGUAGE_ID_Attr, SelectQuery.EQ_Operator, languageId.toString());
								}
							}
						}
					}
				}
				else if (ACTION_PARAMETER_INVENTORY_QUANTITY.equals(parameterName)) {
					int queryOperator = SelectQuery.EQ_Operator;
					Action.Parameter[] subParameters = parameter.getSubParameters();
					if (subParameters != null) {
						for (int j=0; j<subParameters.length; j++) {
							Action.Parameter subParameter = subParameters[j];
							if (ACTION_PARAMETER_COMPARISON_OPERATOR.equals(subParameter.getName())) {
								queryOperator = getSelectQueryOperator(subParameter.getValue());
							}
						}
					}
					_selectQuery.addSelectAttribute(SelectQuery.INVSTVW_QUANTITY_AVAILABLE_Attr, queryOperator, parameterValue);
				}
				else if (ACTION_PARAMETER_LIST_PRICE.equals(parameterName) || ACTION_PARAMETER_OFFER_PRICE.equals(parameterName)) {
					int queryOperator = SelectQuery.EQ_Operator;
					Action.Parameter[] subParameters = parameter.getSubParameters();
					if (subParameters != null) {
						for (int j=0; j<subParameters.length; j++) {
							Action.Parameter subParameter = subParameters[j];
							if (ACTION_PARAMETER_COMPARISON_OPERATOR.equals(subParameter.getName())) {
								queryOperator = getSelectQueryOperator(subParameter.getValue());
							}
							else if (ACTION_PARAMETER_CURRENCY.equals(subParameter.getName())) {
								_selectQuery.addSelectAttribute(SelectQuery.OFFERPRICE_CURRENCY_Attr, SelectQuery.EQ_Operator, subParameter.getValue());
							}
						}
					}
					_selectQuery.addSelectAttribute(SelectQuery.OFFERPRICE_PRICE_Attr, queryOperator, parameterValue);
				}
				else if (ACTION_PARAMETER_AVAILABILITY_DATE.equals(parameterName)) {
					int queryOperator = SelectQuery.EQ_Operator;
					Action.Parameter[] subParameters = parameter.getSubParameters();
					if (subParameters != null) {
						for (int j=0; j<subParameters.length; j++) {
							Action.Parameter subParameter = subParameters[j];
							if (ACTION_PARAMETER_COMPARISON_OPERATOR.equals(subParameter.getName())) {
								queryOperator = getSelectQueryOperator(subParameter.getValue());
							}
						}
					}
					_selectQuery.addSelectAttribute(SelectQuery.CATENTDESC_AVAILABILITY_DATE_Attr, queryOperator, parameterValue + "-00.00.00");
				}
			}
			catch (Exception e) {
			}
		}
	}
  }

  /**
   * Populate the results on catalog entry.
   *
   * @param initiativeId  The ID of the campaign initiative.
   *
   * @return <code>true</code> if results are returned.
   */
  protected boolean populateCatalogEntryResults (Integer initiativeId) {
	boolean results = false;
	try {
		Vector selectQueryCatalogEntryIds = _selectQuery.execute(_maximumNumberOfCatalogEntries, true);
		for (int i=0; i<selectQueryCatalogEntryIds.size(); i++) {
			addCatalogEntry(selectQueryCatalogEntryIds.elementAt(i).toString(), initiativeId);
			results = true;
		}
	}
	catch (Exception e) {
	}
	if (_catalogEntryIds != null) {
		for (int i=0; i<_catalogEntryIds.size(); i++) {
			addCatalogEntry(_catalogEntryIds.elementAt(i).toString(), initiativeId);
			results = true;
		}
	}
	return results;
  }

  /**
   * Populate the results on category.
   *
   * @param initiativeId  The ID of the campaign initiative.
   *
   * @return <code>true</code> if results are returned.
   */
  protected boolean populateCategoryResults (Integer initiativeId) {
	boolean results = false;
	try {
		if (_categoryNames != null) {
			Vector categoryVector = new Vector();
			Integer storeId = getCommandContext().getStoreId();
			Vector memberIds = new Vector();
			Enumeration list = new CatalogAccessBean().findByStoreId(storeId);
			while (list.hasMoreElements()) {
				CatalogAccessBean catalogAccessBean = (CatalogAccessBean) list.nextElement();
				memberIds.addElement(catalogAccessBean.getMemberIdInEJBType());
			}
			for (int i=0; i<_categoryNames.size(); i++) {
				addCategory((String) _categoryNames.elementAt(i), initiativeId, memberIds);
				results = true;
			}
		}
	}
	catch (Exception e) {
	}
	return results;
  }

  /**
   * Populate the results on ad copy.
   *
   * @param initiativeId  The ID of the campaign initiative.
   *
   * @return <code>true</code> if results are returned.
   */
  protected boolean populateCollateralResults (Integer initiativeId) {
	boolean results = false;
	try {
		if (_collateralNames != null) {
			Integer languageId = getCommandContext().getLanguageId();
			Integer storeId = getCommandContext().getStoreId();
			for (int i=0; i<_collateralNames.size(); i++) {
				addCollateral((String) _collateralNames.elementAt(i), initiativeId, storeId, languageId);
				results = true;
			}
		}
	}
	catch (Exception e) {
	}
	return results;
  }

  /**
   * Populate the results on all e-marketing spot display types.
   *
   * @param initiativeId  The ID of the campaign initiative.
   *
   * @return <code>true</code> if results are returned.
   */
  protected boolean populateResults (Integer initiativeId) {
	boolean results = false;
	if (populateCatalogEntryResults(initiativeId))
	 {
	  results = true;
	 }
	if (populateCollateralResults(initiativeId))
	 {
	  results = true;
	 }
	if (populateCategoryResults(initiativeId))
	 {
	  results = true;
	 }
	return results;
  }

  /**
   * Sets the name of the e-marketing spot.
   *
   * @param eMarketingSpotName  The name of the e-marketing spot.
   */
  public void setEMarketingSpotName (String eMarketingSpotName) {
	_eMarketingSpotName = eMarketingSpotName;
  }

  /**
   * Sets the maximum number of catalog entries can be displayed in an e-marketing spot.
   *
   * @param maximumNumberOfCatalogEntries  The maximum number of catalog entries can be displayed in an e-marketing spot.
   */
  public void setMaximumNumberOfCatalogEntries (int maximumNumberOfCatalogEntries) {
	_maximumNumberOfCatalogEntries = maximumNumberOfCatalogEntries;
  }

  /**
   * Sets the maximum number of categories can be displayed in an e-marketing spot.
   *
   * @param maximumNumberOfCategories  The maximum number of categories can be displayed in an e-marketing spot.
   */
  public void setMaximumNumberOfCategories (int maximumNumberOfCategories) {
	_maximumNumberOfCategories = maximumNumberOfCategories;
  }

  /**
   * Sets the maximum number of ad copies can be displayed in an e-marketing spot.
   *
   * @param maximumNumberOfCollateral  The maximum number of ad copies can be displayed in an e-marketing spot.
   */
  public void setMaximumNumberOfCollateral (int maximumNumberOfCollateral) {
	_maximumNumberOfCollateral = maximumNumberOfCollateral;
  }

  /**
   * Gets the instance of the likemind server.
   *
   * @return An instance of the likemind server.
   */
  protected WCSRecommendationServer getWCSRecommendationServer()
   {
    return new LMRecommender();
   }

  /**
   * The appendCampaignLogResults method writes the execution log of the campaign initiative.
   *
   * @param object  The campaign initiative result log.
   */
  protected void appendCampaignLogResults(Object object)
   {
    if (_campaignLogResults == null)
     {
      _campaignLogResults = "";
     }
    else
     {
      _campaignLogResults += ",";
     }
     _campaignLogResults += object;
   }
}